/****************************************************
* Dmodulation CLIP avec MSP430F1132                *
****************************************************/

/*------------------------------------------------------------------------------------
  Nom          :  CLIP_1232_Nokia_3310.h
  Description  :  Fichier "entte" pour carte CLIP de CREMMEL M. avec LCD Nokia 3310
  Author       :  08/05/2006 - CREMMEL Marcel
  History      :  08/05/2006 - Premire version
--------------------------------------------------------------------------------------*/
#include <msp430x12x2.h>

/*------------------------------
* Indicateurs du registre R4
--------------------------------*/
#define WAIT_SMMR        1  // Indique la phase d'attente du signal SMMR
#define WAIT_MARK        2  // Indique la phase d'attente du signal MARK
#define WAIT_MSG         4  // Indique la phase de rception du message
#define NEW_MSG          8  // Indique un nouveau message
#define TIME_OUT         16 // Indique un problme pdt la rception de la trame
#define LAST_RXD         32 // Etat prcdent de RXD

/*------------------------------------------------------------------------------------
                                         Constantes
--------------------------------------------------------------------------------------*/

/*------------------
       Cblage
--------------------*/
// Ports SPI
#define DIN_SPI     BIT1  //  P3.1 = SOMI = SDIN
#define CLK_SPI     BIT3  //  P3.3 = UCLK = SCLK

// Connexions LCD Nokia (transmission via SPI)
#define Lcd_CE_Pin  BIT0  //  P3.0 = CE_LCD
#define Lcd_DC_Pin  BIT2  //  P3.2 = D/C
#define Lcd_RST_Pin BIT2  //  P2.2 = RESET_LCD

// Connexions CNA MAX550BCPA (transmission via SPI)
#define CS_CNA   BIT6   // P3.6 = CS_CNA, actif  "0"
#define LDAC     BIT4   // P3.4 = LDAC, actif  "0"

// Sortie RXD dmodule
#define RXD      BIT7   // P3.7 = sortie dmodule RXD

// Autre signaux
#define PWR_ON   BIT0   // P2.0 = PWR_ON (alim Aops)
#define SONNE    BIT5   // P2.5 = SONNE (entre dtection de sonnerie)

// Signaux de test
#define TA1      BIT2   // P1.2 = signal de test TA1

/*-------------------------
  Constantes LCD Nokia 3310
---------------------------*/
#define Lcd_X_res       84
#define Lcd_Y_res       48
// Commandes controleur PCD8544
#define Function_Set    0x20    //Bit 2 = PD, Bit 1 = V, Bit 1 = H
#define Lcd_ON_Norm_H   0x20    //Lcd actif, adressage H, mode normal
#define Lcd_ON_Norm_V   0x22    //Lcd actif, adressage V, mode normal
#define Lcd_ON_Etendu_H 0x21    //Lcd actif, adressage H, mode tendu (H=1)
#define Lcd_OFF_Norm_H  0x24    //Lcd "power down", adressage H, mode normal
// Commandes normales (bit H="0")
#define Set_Config      0x08    //Bit 2 = D, bit 0 = E
#define Set_Adr_Y       0x40    //Adresse Y dans 3 bits LSB (0  5)
#define Set_Adr_X       0x80    //Adresse X dans 7 bits LSB (0  83)
// Commandes tendues (bit H="1")
#define Set_Bias        0x10    //Bias Mode dans bits 2  0
#define Set_Vop         0x80    //Vop dans bits 6  0
#define Set_TC          0x04    //Coef de temprature dans bits 1  0
// Positions des paramtres sur le LCD
#define Pos_FRX_X       5       //Position X de la frquence de rception (0  83)
#define Pos_FRX_Y       2       //Position Y de la frquence de rception (0  5)
#define Pos_Filtre_X    41      //Position X de la frquence de rception (0  83)
#define Pos_Filtre_Y    4       //Position Y de la frquence de rception (0  5)

/*------------------
  Constantes Timer A
--------------------*/
// Interruption Timer A, voie 0
#define Pulse_OUT1      100     //Dure de l'impulsion de dclenchement du CAN
#define Periode_IntTA0  588     //Soit une frquence de 6800Hz avec horloge de 4MHZ

/*---------------
  Constantes UART
-----------------*/
#define RXBufSize 32 // Taille du buffer de rception (puissance de 2)
