/****************************************************
* Dmodulation CLIP avec MSP430F1132                *
****************************************************/

/* Le programme d'interruption Timer A (OC 0) est dcrit dans le module
   TA0_int_CLIP.s43.
   Il comporte aussi la dclaration de son vecteur d'interruption */

/*------------------------------------------------------------------------------------
  Nom          :  Demod_CLIP_1232_Nokia.c
  Description  :  Dmodulation CLIP
  Author       :  18/04/2006 - CREMMEL Marcel
  History      :  12/04/2006 - Premire version
--------------------------------------------------------------------------------------*/
#include <msp430x12x2.h>
#include <stdbool.h> // Pour le type "bool"
#include <stdlib.h>  // Pour le type div_t

__regvar __no_init int IND_R4 @ __R4; // Rservation registre R4
/*------------------------------
* Indicateurs du registre IND_R4
--------------------------------*/
#define WAIT_SMMR        1  // Indique la phase d'attente du signal SMMR
#define WAIT_MARK        2  // Indique la phase d'attente du signal MARK
#define WAIT_MSG         4  // Indique la phase de rception du message
#define NEW_MSG          8  // Indique un nouveau message
#define TIME_OUT         16 // Indique un problme pdt la rception de la trame
#define LAST_RXD         32 // Etat prcdent de RXD

/*------------------------------------------------------------------------------------
                                         Constantes
--------------------------------------------------------------------------------------*/

/*------------------
       Cblage
--------------------*/
// Connexions LCD Nokia
#define Lcd_CE_Pin   BIT0  //  P3.0 = CE_LCD
#define Lcd_DIN_Pin  BIT1  //  P3.1 = SOMI = SDIN
#define Lcd_DC_Pin   BIT2  //  P3.2 = D/C
#define Lcd_CLK_Pin  BIT3  //  P3.3 = UCLK = SCLK
#define Lcd_RST_Pin  BIT2  //  P2.2 = RESET_LCD

// Connexions CNA ZN426E
#define ZN426E   P3OUT  //  Les 7 bits du port 3

// Sortie RXD dmodule
//#define RXD      BIT7   // P3.7 = sortie dmodule RXD
#define RXD      BIT3   // P2.3 sortie dmodule RXD sur plaquette

// CNA MAX550 (SCLK et DIN communs avec LCD)
#define CS_CNA   BIT6   // P3.6 = CS_CNA
#define LDAC     BIT4   // P3.4 = LDAC

// Autre signaux
#define PWR_ON   BIT0   // P2.0 = PWR_ON (alim Aops)
#define SONNE    BIT5   // P2.5 = SONNE (entre dtection de sonnerie)

// Signaux de test
#define TEST1    BIT0   // P1.0 = signal de test
#define TEST2    BIT1   // P1.1 = signal de test
#define TEST3    BIT2   // P1.2 = signal de test
#define TEST4    BIT7   // P3.7 = signal de test

// Calibration de la frquence des interruptions TimerA0 (6800Hz)
// Cblage : sortie UCLK=P3.3 relie sur entre P1.3=CCI2A

/*------------------
  Constantes Timer A
--------------------*/
// Interruption Timer A, voie 0
#define Pulse_OUT1      100     //Dure de l'impulsion de dclenchement du CAN
#define Periode_IntTA0  588     //Soit une frquence de 6800Hz avec horloge de 4MHZ

/*---------------
  Constantes UART
-----------------*/
#define RXBufSize 32 // Taille du buffer de rception (puissance de 2)

/*---------------------------------------
  Commandes du controleur Philips PCF8833
---------------------------------------*/
#define NO_OPERATION        0x00
#define SOFT_RESET          0x01
#define BOOSTER_OFF         0x02
#define BOOSTER_ON          0x03
#define DISPLAY_INFO        0x04  // read 25bits where MSB are dummy trash
#define DISPLAY_STATUS      0x09  // read 33bits where MSB are dummy trash
#define SLEEP_IN            0x10
#define SLEEP_OUT           0x11
#define PARTIAL_MODE_ON     0x12
#define NORMAL_MODE_ON      0x13
#define INVERSION_OFF       0x20
#define INVERSION_ON        0x21
#define PIXELS_OFF          0x22
#define PIXELS_ON           0x23
#define SET_CONTRAST        0x25
#define DISPLAY_OFF         0x28
#define DISPLAY_ON          0x29
#define SET_X_ADDR          0x2A // ATENTION! when X,Y address are changed BOTH must be changed
  // START_X                0xXX
  // STOP_X                 0xXX
#define SET_Y_ADDR          0x2B
  // START_Y                0xXX
  // STOP_Y                 0xXX
#define MEM_WRITE           0x2C
  // x times pixel data     0xXX
#define COLOR_SET           0x2D
  // 8 times red palette    0xXX
  // 8 times green palette  0xXX
  // 4 times blue palette   0xXX
#define PARTIAL_AREA        0x30
  // START_Y                0xXX
  // STOP_Y                 0xXX
#define SCROLL_DEFINITON    0x33 // always vertical
  // TOP_FIXED_LINES        0xXX
  // SCROLL_LINES           0xXX
  // BOTTOM_FIXED_LINES     0xXX
#define SET_SCROLL          0x37 // SEP scroll entry point
  // ROW_Y                  0xxx
#define MEM_CONTROL         0x36
#define   MEM_RGB           0x08 // "0": ordre RGB, "1": BGR
#define   MEM_LAO           0x10 // "0":haut vers bas, "1":bas vers haut
#define   MEM_VW            0x20 // "0":criture ds le sens X
                                  // "1":criture ds le sens Y
#define   MEM_MX            0x40 // "0":normal, "1":miroir en X
#define   MEM_MY            0x80 // "0":normal, "1":miroir en Y
#define IDLE_OFF            0x38
#define IDLE_ON             0x39
#define COLOR_INTERFACE     0x3A
  // pixelformat            0xXX
#define   COLOR_8_BIT       0x02
#define   COLOR_12_BIT      0x03
#define   COLOR_16_BIT      0x05
#define SET_VOP             0xB0 // irrelevant for Nokia6100
  // 2x                     0xXX
#define BOTTOM_ROW_SWAP_OFF 0xB4
#define BOTTOM_ROW_SWAP_ON  0xB5
#define TOP_ROW_SWAP_OFF    0xB6
#define TOP_ROW_SWAP_ON     0xB7
#define SUPER_FRAME_INV_OFF 0xB8
#define SUPER_FRAME_INV_ON  0xB9
#define DATA_ORDER_MSB      0xBA
#define DATA_ORDER_LSB      0xBB
#define TEMP_READ           0xC8 // read

/*-------------------------------
  Couleurs (palette 256 couleurs)
---------------------------------*/
#define Noir  0
#define Blanc 255
#define Bleu  3
#define Rouge 7*32
#define Vert  7*4
#define Jaune Rouge+Vert

/*------------------------------------------------------------------------------------
                                    Dfinition de type
--------------------------------------------------------------------------------------*/
typedef enum
{
    B1200 = 0,
    B2400 = 1,
    B4800 = 2,
    B9600 = 3
} BaudRate; // Vitesse de transmission UART

/*------------------------------------------------------------------------------------
                                    Variables globales
-------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------
   Variables du programme d'interruption "TA0_int_CLIP.s43"
-------------------------------------------------------------
 Elles sont dclares ds le mme ordre pour obtenir les mmes adresses
-----------------
   Filtre passe-bas
-----------------*/
int WDF_PARMS[5];  // Pile du filtre passe-bas
#define IN_Z    0
#define Z1_1    2
#define Z1_2    4
#define Z3_1    6
#define Z3_2    8
int end_of_parms; //
int Count_Bit;    // Compteur d'chantillons  "1" pour dtecter dbut trame
int R6_T;         // Fonction "retard"
/*-------------------------
  Compteurs d'interruptions
---------------------------*/
int Compt_Int_6800; // Permet d'arrter la procdure de rception de la trame CLIP
                    // en cas de problme (pas de trame, perturbations diverses, ...)
int Compt_Sonnerie; // Pour dclencher la procdure de rception de la trame CLIP
                    // qu'aprs un signal de sonnerie
/*-----------------
       UART
-------------------*/
// Index en rception :
int RXReadIndex, RXWriteIndex;
// Comptage des octets reus :
char RXCharCount;
char Lng_Message; // Pour mmoriser la longueur des messages
// Buffer de rception
char RXBuffer[RXBufSize];
/*-----------------
        LCD
-------------------*/
// Paramtres pour controleur Philips
char Texte_MEM_CONTROL; //Paramtre de MEM_CONTROL pour le texte
char Image_MEM_CONTROL; //Paramtre de MEM_CONTROL pour l'image
// Paramtres de texte
char Couleur_texte; //Code de couleur du texte
char Fond_texte;    //Code de couleur du fond
char Sens_texte; //Sens d'criture du texte : 0:0, 1:90, 2:180 3:270
/*-----------------------------------------------------------
                     Autres variables
-------------------------------------------------------------*/
char Pos_Y_Msg; //Position Y des messages reus
char Premier_Numero; //Indique la rception du 1 numro

/*------------------------------------------------------------------------------------
                                 Constantes en mmoire flash
-------------------------------------------------------------------------------------*/
/* ---------------------
   Polices de caractres
   ---------------------
  ------
  8 x 11
  ------
   Taille en octets    : 0x02AC, 684
   Largeur police      : 8
   Hauteur police      : 11
   Code du 1 caractre: 0x20
   Code du dernier car.: 0x7E
   Bits par pixel      : 1
*/
typedef struct
{
  unsigned int  Size;
  unsigned char Width; //Pour espacement constant
  unsigned char Heigth;
  unsigned char Bits_per_Pixels;
  // if MSB are set then font is a compressed font
  unsigned char First_Char;
  unsigned char Last_Char;
  unsigned char Char_Widths[0x7E - 0x20 +1];// Largeurs de chaque caractre
  unsigned char Font_data[582];// Description des caractres (suite de pixels sans trou)
} Tfont8x11;

const Tfont8x11 Font8x11 =
 { 0x02AC,
   0x08,
   0x0B,
   0x01,
   0x20,
   0x7E,
   {
    0x03, 0x01, 0x04, 0x06, 0x05, 0x07, 0x05, 0x01, 0x02, 0x02, 0x03, 0x05, 0x02, 0x02, 0x01, 0x04,
    0x05, 0x03, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x01, 0x02, 0x04, 0x05, 0x04, 0x05,
    0x07, 0x07, 0x05, 0x06, 0x06, 0x05, 0x05, 0x06, 0x06, 0x01, 0x04, 0x06, 0x05, 0x07, 0x06, 0x06,
    0x06, 0x06, 0x06, 0x05, 0x05, 0x06, 0x07, 0x07, 0x07, 0x07, 0x07, 0x02, 0x04, 0x02, 0x05, 0x06,
    0x02, 0x05, 0x05, 0x05, 0x05, 0x05, 0x02, 0x05, 0x05, 0x01, 0x01, 0x05, 0x01, 0x07, 0x05, 0x05,
    0x05, 0x05, 0x02, 0x04, 0x02, 0x05, 0x05, 0x07, 0x04, 0x05, 0x04, 0x03, 0x01, 0x03, 0x06
   },
   {
    0x00, 0x00, 0x00, 0x00, 0xFE, 0x72, 0x00, 0x00, 0x00, 0xE0, 0x00, 0x44, 0xF8, 0x0F, 0x11, 0x88,
    0xF0, 0x1F, 0x22, 0x30, 0x42, 0x22, 0xFF, 0x13, 0x09, 0x31, 0x04, 0x51, 0x84, 0x12, 0x48, 0x02,
    0x29, 0x44, 0x11, 0x84, 0x39, 0x32, 0x62, 0x12, 0x60, 0x80, 0xE4, 0x00, 0xFE, 0x09, 0x50, 0x80,
    0xFC, 0xA3, 0x00, 0x02, 0x28, 0x00, 0x04, 0x20, 0xC0, 0x07, 0x08, 0x40, 0x00, 0x20, 0x80, 0x80,
    0x00, 0x04, 0x00, 0x01, 0x0C, 0x18, 0x30, 0x60, 0x00, 0x7F, 0x04, 0x24, 0x20, 0x01, 0xF1, 0x87,
    0x00, 0x04, 0xF0, 0x1F, 0xC1, 0x04, 0x25, 0x24, 0x11, 0x71, 0x88, 0x20, 0x02, 0x12, 0x91, 0x88,
    0xB8, 0x03, 0x0C, 0x58, 0x30, 0xC2, 0x7F, 0x80, 0xF0, 0x89, 0x84, 0x24, 0x24, 0x21, 0xF1, 0xF0,
    0x47, 0x44, 0x22, 0x12, 0x11, 0x71, 0x04, 0x20, 0x38, 0x31, 0x68, 0xC0, 0x00, 0xDC, 0x11, 0x91,
    0x88, 0x44, 0xC4, 0x1D, 0x8E, 0x88, 0x48, 0x44, 0x22, 0xE2, 0x0F, 0x84, 0x00, 0x08, 0x21, 0x20,
    0x80, 0x02, 0x22, 0x08, 0x02, 0x05, 0x28, 0x40, 0x01, 0x0A, 0x50, 0x20, 0x08, 0x22, 0xA0, 0x00,
    0x02, 0x01, 0x04, 0x20, 0x2C, 0x11, 0x70, 0x00, 0x3C, 0x10, 0x42, 0x26, 0x4A, 0xD1, 0x09, 0x51,
    0x70, 0x00, 0x0C, 0x1C, 0x98, 0x30, 0x04, 0x26, 0xC0, 0x01, 0x30, 0xFF, 0x89, 0x48, 0x44, 0x22,
    0xE2, 0x0E, 0x7F, 0x04, 0x24, 0x20, 0x01, 0x09, 0x88, 0x20, 0xFE, 0x13, 0x90, 0x80, 0x04, 0x44,
    0x10, 0x7C, 0xF8, 0x4F, 0x44, 0x22, 0x12, 0x91, 0x80, 0xFC, 0x27, 0x02, 0x11, 0x88, 0x40, 0x00,
    0xFC, 0x11, 0x90, 0x80, 0x44, 0x24, 0x12, 0xF2, 0xF9, 0x0F, 0x04, 0x20, 0x00, 0x01, 0x08, 0xFC,
    0xE7, 0x3F, 0xC0, 0x00, 0x08, 0x40, 0xFE, 0xF1, 0x1F, 0x0C, 0x90, 0x40, 0x08, 0x81, 0x00, 0xC8,
    0x7F, 0x00, 0x02, 0x10, 0x80, 0x00, 0xE4, 0x3F, 0x0C, 0x80, 0x01, 0x30, 0x60, 0xC0, 0x80, 0xFF,
    0xFC, 0xC7, 0x00, 0x18, 0x00, 0x01, 0x30, 0xFE, 0xE3, 0x8F, 0x80, 0x04, 0x24, 0x20, 0x01, 0xF1,
    0xC7, 0x7F, 0x22, 0x10, 0x81, 0x08, 0x44, 0xC0, 0x01, 0xFE, 0x08, 0x48, 0x40, 0x82, 0x12, 0x18,
    0x7F, 0xFD, 0x27, 0x02, 0x11, 0x88, 0x40, 0x04, 0xDC, 0xE3, 0x88, 0x88, 0x44, 0x24, 0x22, 0xE2,
    0x08, 0x40, 0x00, 0xFE, 0x13, 0x80, 0x00, 0xFC, 0x03, 0x20, 0x00, 0x01, 0x08, 0x40, 0xFE, 0x31,
    0x00, 0x0E, 0x80, 0x01, 0x30, 0x60, 0xE0, 0xC0, 0x00, 0xFE, 0x00, 0x18, 0x30, 0x70, 0x00, 0x0C,
    0x80, 0xF9, 0xC3, 0x60, 0x88, 0x80, 0x02, 0x08, 0xA0, 0x80, 0x08, 0x83, 0x19, 0x00, 0x01, 0x10,
    0x00, 0x1F, 0x04, 0x10, 0x60, 0x00, 0x81, 0x09, 0x4A, 0x48, 0x22, 0x92, 0x90, 0x82, 0x0C, 0xE4,
    0x7F, 0x01, 0x32, 0x00, 0x06, 0xC0, 0x00, 0x98, 0x00, 0xFD, 0x8F, 0x00, 0x02, 0x08, 0x80, 0x00,
    0x08, 0x00, 0x20, 0x00, 0x01, 0x08, 0x40, 0x00, 0x02, 0x50, 0x00, 0x04, 0x00, 0x0C, 0x94, 0xA0,
    0x04, 0x25, 0xF0, 0xF9, 0x0F, 0x42, 0x10, 0x82, 0x10, 0x78, 0xC0, 0x03, 0x21, 0x08, 0x41, 0x08,
    0x24, 0xE0, 0x81, 0x10, 0x84, 0x20, 0xE4, 0x3F, 0xF0, 0x40, 0x09, 0x4A, 0x50, 0x02, 0x0B, 0xFF,
    0x24, 0x00, 0x9E, 0x08, 0x45, 0x28, 0x42, 0xF1, 0xF7, 0x1F, 0x08, 0x20, 0x00, 0x01, 0xF0, 0xC9,
    0x4F, 0xFE, 0xFF, 0x03, 0x02, 0x28, 0x20, 0x02, 0x20, 0xFF, 0xC1, 0x0F, 0x02, 0x10, 0x00, 0x1F,
    0x04, 0x20, 0x00, 0x3E, 0xF8, 0x81, 0x00, 0x02, 0x10, 0x00, 0x1F, 0x78, 0x20, 0x04, 0x21, 0x08,
    0x81, 0x07, 0xFE, 0x11, 0x82, 0x10, 0x84, 0xC0, 0x03, 0x1E, 0x08, 0x41, 0x08, 0x42, 0xF0, 0x8F,
    0x1F, 0x04, 0x40, 0x02, 0x25, 0x48, 0x81, 0x84, 0x3F, 0x10, 0x82, 0x0F, 0x80, 0x00, 0x04, 0x10,
    0xF8, 0xC1, 0x00, 0x18, 0x00, 0x03, 0x06, 0x0C, 0xE0, 0x01, 0x30, 0x60, 0xC0, 0x00, 0x18, 0x00,
    0x83, 0x07, 0xCC, 0x80, 0x01, 0x0C, 0x98, 0x01, 0x20, 0x1E, 0x01, 0x07, 0x08, 0x3C, 0x20, 0x06,
    0x29, 0x28, 0xC1, 0x08, 0x08, 0xBC, 0x17, 0xC0, 0xFF, 0x05, 0xD0, 0x7B, 0x20, 0x20, 0x80, 0x00,
    0x04, 0x40, 0x00, 0x02, 0x08, 0x00}
 };

/*-------
  13 x 16
  -------
   Taille en octets    : 0x8E = 142
   Largeur police      : 13
   Hauteur police      : 16
   Code du 1 caractre: 0x30
   Code du dernier car.: 0x39
   Bits par pixel      : 1
*/

typedef struct
{
  unsigned int  Size;
  unsigned char Width; //Pour espacement constant
  unsigned char Heigth;
  unsigned char Bits_per_Pixels;
  // if MSB are set then font is a compressed font
  unsigned char First_Char;
  unsigned char Last_Char;
  unsigned char Char_Widths[0x39 - 0x30 +1];// Largeurs de chaque caractre
  unsigned char Font_data[152];// Description des caractres (suite de pixels sans trou)
} Tfont13x16;

const Tfont13x16 Font13x16 =
 {0x008E,
  0x09,
  0x10,
  0x01,
  0x30,
  0x39,
  { 0x08, 0x04, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08},
  {
    0xF0, 0x03, 0xFC, 0x0F, 0x0E, 0x1C, 0x02, 0x10, 0x02, 0x10, 0x0E, 0x1C, 0xFC, 0x0F, 0xF0, 0x03,
    0x04, 0x00, 0x04, 0x00, 0xFE, 0x1F, 0xFE, 0x1F, 0x18, 0x18, 0x1C, 0x1C, 0x06, 0x16, 0x02, 0x13,
    0x82, 0x11, 0xC6, 0x10, 0x7C, 0x10, 0x38, 0x10, 0x18, 0x06, 0x1C, 0x0E, 0x46, 0x18, 0x42, 0x10,
    0x42, 0x10, 0xE6, 0x18, 0xBC, 0x0F, 0x18, 0x07, 0x00, 0x03, 0x80, 0x03, 0xE0, 0x02, 0x70, 0x02,
    0x1C, 0x02, 0xFE, 0x1F, 0xFE, 0x1F, 0x00, 0x02, 0xFE, 0x06, 0xFE, 0x0E, 0x62, 0x18, 0x22, 0x10,
    0x22, 0x10, 0x62, 0x18, 0xC2, 0x0F, 0x82, 0x07, 0xF8, 0x07, 0xFC, 0x0F, 0x66, 0x18, 0x22, 0x10,
    0x22, 0x10, 0x66, 0x18, 0xCC, 0x0F, 0x88, 0x07, 0x02, 0x00, 0x02, 0x00, 0x02, 0x00, 0x02, 0x1E,
    0xC2, 0x1F, 0xF2, 0x01, 0x3E, 0x00, 0x0E, 0x00, 0x18, 0x07, 0xBC, 0x0F, 0xE6, 0x18, 0x42, 0x10,
    0x42, 0x10, 0xE6, 0x18, 0xBC, 0x0F, 0x18, 0x07, 0x78, 0x04, 0xFC, 0x0C, 0x86, 0x19, 0x02, 0x11,
    0x02, 0x11, 0x86, 0x19, 0xFC, 0x0F, 0xF8, 0x07
  }
 };

/*------------------------------------------------------------------------------------
                                        HORLOGES
---------------------------------------------------------------------------------------*/

/*--------------------------------------------------
  Nom          :  SetDCOClock
  Description  :  Affectation frquence DCO
  Arguments    :  DCO : valeur des 3 bits DCO
                  RES : valeur des 3 bits RESEL
  Valeur renvoye : aucune.
----------------------------------------------------*/
void SetDCOClock(int DCO,int RES)
{
  DCOCTL=(DCOCTL & ~(DCO2|DCO1|DCO0)) | (DCO << 5); //Affectation DCO[2..0]
  BCSCTL1=(BCSCTL1 & ~(RSEL2|RSEL1|RSEL0)) | RES  ; //Affectation RESEL[2..0]
}

/*------------------------------------------------------------------------------------
                                   Fonctions de gestion du LCD
--------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------
  Nom          :  InitSPI_Lcd
  Description  :  Initialisation du coupleur SPI
                  Il est utilis pour la transmission vers le LCD
  Arguments    :  aucun
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void InitSPI_Lcd(void)
{
  U0CTL=SWRST; //SWRST = "1" pour configurer l'USART en mode SPI
  U0CTL=SWRST|CHAR|SYNC|MM; //Mode Master SPI 8 bits
  U0TCTL=CKPH|SSEL1|STC; //CKPH/CKPL=10 : format PCD8544
                         //SSEL=10 : SMCLK
                         //STC=1 : STE inhib
  U0BR0=4; //
  U0BR1=0; // U0BR = 4 : Baud rate = SMCLK/4
  U0MCTL=0; //Impos en mode SPI
  ME2|=USPIE0; //Validation SPI
  U0CTL&=~SWRST; //SWRST = "0" fin de configuration
  P3DIR|=Lcd_CE_Pin|Lcd_DIN_Pin|Lcd_CLK_Pin|Lcd_DC_Pin;
  P2DIR|=Lcd_RST_Pin;
  P3SEL =Lcd_DIN_Pin|Lcd_CLK_Pin;
  // Slection du controleur (actif  "0").
  P3OUT |=  Lcd_CE_Pin;//CE = "1"
//  P3OUT &= ~Lcd_CE_Pin;//CE = "0"
  // Pas de reset (RST = "1")
  P2OUT|=Lcd_RST_Pin;  //RST = "1"
}

/*---------------------------------------------------------------
  Nom          :  LcdSend_Cmd
  Description  :  Envoi d'une commande vers le controleur du LCD via coupleur SPI
                  Premier bit (D/C) transmis hors SPI car trame de 9 bits
  Arguments    :  data -> Commande  transmettre
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void LcdSend_Cmd(char Data)
{
  // Attendre fin de transmission
  while ((U0TCTL & TXEPT)==0) {};
  // Affectation signal D/C (1 impulsion de la trame 9 bits)
  P3OUT |=  Lcd_CE_Pin;//CE = "1"
  P3SEL&=~(Lcd_DIN_Pin|Lcd_CLK_Pin);//Gestion LCD sans coupleur SPI
  P3OUT &= ~Lcd_CE_Pin;//CE = "0"
  P3OUT &= ~Lcd_DIN_Pin; // D/C = 0 : commande
  P3OUT |=  Lcd_CLK_Pin; // Premire impulsion CLK
  P3OUT &= ~Lcd_CLK_Pin;
  P3SEL |= Lcd_DIN_Pin|Lcd_CLK_Pin;//Gestion LCD avec coupleur SPI
  // Transmission srie de l'octet (8 derniers bits)
  U0TXBUF=Data;
}

/*---------------------------------------------------------------
  Nom          :  LcdSend_Data
  Description  :  Envoi d'une donne vers le controleur du LCD via coupleur SPI
                  Premier bit (D/C) transmis hors SPI car trame de 9 bits
  Arguments    :  data -> Donne  transmettre
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void LcdSend_Data(char Data)
{
  // Attendre fin de transmission
  while ((U0TCTL & TXEPT)==0) {};
  // Affectation signal D/C (1 impulsion de la trame 9 bits)
  P3OUT |=  Lcd_CE_Pin;//CE = "1"
  P3SEL&=~(Lcd_DIN_Pin|Lcd_CLK_Pin);//Gestion LCD sans coupleur SPI
  P3OUT &= ~Lcd_CE_Pin;//CE = "0"
  P3OUT |= Lcd_DIN_Pin; // D/C = 1 : donne
  P3OUT |= Lcd_CLK_Pin; // Premire impulsion CLK
  P3OUT &= ~Lcd_CLK_Pin;
  P3SEL |= Lcd_DIN_Pin|Lcd_CLK_Pin;//Gestion LCD avec coupleur SPI
  // Transmission srie de l'octet (8 derniers bits)
  U0TXBUF=Data;
}

/*---------------------------------------------------------------
  Nom          :  Delay
  Description  :  Production d'un dlai logiciel
  Arguments    :  aucun
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void Delay(int Delai)
{
  int i;
  for ( i = -Delai; i < Delai; i++ );
}

/*---------------------------------------------------------------
  Nom          :  LcdLittleChr
  Description  :  Dessin d'un caractre de la police 8x11
  Arguments    :  x, y : coordonnes
                  ch : code ascii du caractre
  Valeur renvoye : abscisse x du prochain caractre
-----------------------------------------------------------------*/
char LcdLittleChr(char x, char y, char ch)
{
 int index, i, j, k, t;
 int index_byte, index_pixel;
 char Cmd_H, Cmd_V;
 char Hor, Ver;
 index=ch-0x20;
 if (index < 0) return 0;
 if (Sens_texte & 1) //Sens du texte ?
    { Cmd_H=SET_Y_ADDR; Cmd_V=SET_X_ADDR;// Sens Y
      Hor=y; Ver=x; }
 else
    { Cmd_H=SET_X_ADDR; Cmd_V=SET_Y_ADDR;// Sens X
      Hor=x; Ver=y; }
 LcdSend_Cmd(Cmd_V);
  LcdSend_Data(Ver);
  LcdSend_Data(Ver+Font8x11.Heigth-1);
 LcdSend_Cmd(Cmd_H);
  LcdSend_Data(Hor);
  LcdSend_Data(Hor+Font8x11.Char_Widths[index]);
 LcdSend_Cmd(MEM_WRITE);
 i=0;
 k=0;
 //k = nb de pixels pour atteindre le caractre
 for (j=0; j<index; j++) k=k+Font8x11.Char_Widths[j]*Font8x11.Heigth;
 index_byte=(k >> 3);// = k div 8
 index_pixel=k-(index_byte << 3);// = k mod 8
 for (i=0; i<Font8x11.Char_Widths[index]*Font8x11.Heigth; i++)
  {
   t=Font8x11.Font_data[index_byte];
   if ((t & (1 << index_pixel))==0) LcdSend_Data(Fond_texte);
                               else LcdSend_Data(Couleur_texte);
   index_pixel++;
   if (index_pixel==8) {index_pixel=0; index_byte++;}
  }
 for (i=0; i<Font8x11.Heigth; i++) LcdSend_Data(Fond_texte);
 return Font8x11.Char_Widths[index];
}

/*---------------------------------------------------------------
  Nom          :  LcdLittleStr
  Description  :  Dessin d'une chane de caractre ds la police 8x11
  Arguments    :  x, y : coordonnes
                  s : chane du caractre
  Valeur renvoye : aucune
-----------------------------------------------------------------*/
void LcdLittleStr(char x, char y, char *s)
{
 if (*s==0) return;
 LcdSend_Cmd(MEM_CONTROL); LcdSend_Data(Texte_MEM_CONTROL);
 while (*s)
  {
    if (Sens_texte & 1) //Sens du texte ?
         y=y+LcdLittleChr(x,y,*s++)+1; // Sens Y
    else x=x+LcdLittleChr(x,y,*s++)+1; // Sens X
  }
}

/*---------------------------------------------------------------
  Nom          :  LcdBigChr
  Description  :  Dessin d'un caractre de la police 13x16
  Arguments    :  x, y : coordonnes
                  ch : code ascii du caractre
  Valeur renvoye : abscisse x du prochain caractre
-----------------------------------------------------------------*/
char LcdBigChr(char x, char y, char ch)
{
 int index, i, j, k, t;
 int index_byte, index_pixel;
 char Cmd_H, Cmd_V;
 char Hor, Ver;
 index=ch-0x30;
 if (index < 0) return 0;
 if (Sens_texte & 1) //Sens du texte ?
    { Cmd_H=SET_Y_ADDR; Cmd_V=SET_X_ADDR;// Sens Y
      Hor=y; Ver=x; }
 else
    { Cmd_H=SET_X_ADDR; Cmd_V=SET_Y_ADDR;// Sens X
      Hor=x; Ver=y; }
 LcdSend_Cmd(Cmd_V);
  LcdSend_Data(Ver);
  LcdSend_Data(Ver+Font13x16.Heigth-1);
 LcdSend_Cmd(Cmd_H);
  LcdSend_Data(Hor);
  LcdSend_Data(Hor+Font13x16.Char_Widths[index]+1);
 LcdSend_Cmd(MEM_WRITE);
 for (i=0; i<Font13x16.Heigth; i++) LcdSend_Data(Fond_texte);
 i=0;
 k=0;
 //k = nb de pixels pour atteindre le caractre
 for (j=0; j<index; j++) k=k+Font13x16.Char_Widths[j]*Font13x16.Heigth;
 index_byte=(k >> 3);// = k div 8
 index_pixel=k-(index_byte << 3);// = k mod 8
 for (i=0; i<Font13x16.Char_Widths[index]*Font13x16.Heigth; i++)
  {
   t=Font13x16.Font_data[index_byte];
   if ((t & (1 << index_pixel))==0) LcdSend_Data(Fond_texte);
                               else LcdSend_Data(Couleur_texte);
   index_pixel++;
   if (index_pixel==8) {index_pixel=0; index_byte++;}
  }
 for (i=0; i<Font13x16.Heigth; i++) LcdSend_Data(Fond_texte);
 return Font13x16.Char_Widths[index]+1;
}

/*---------------------------------------------------------------
  Nom          :  LcdBigStr
  Description  :  Dessin d'une chane de caractre ds la police 8x11
  Arguments    :  x, y : coordonnes
                  s : chane du caractre
  Valeur renvoye : aucune
-----------------------------------------------------------------*/
void LcdBigStr(char x, char y, char *s)
{
 if (*s==0) return;
 LcdSend_Cmd(MEM_CONTROL); LcdSend_Data(Texte_MEM_CONTROL);
 while (*s)
  {
    if (Sens_texte & 1) //Sens du texte ?
         y=y+LcdBigChr(x,y,*s++)+1; // Sens Y
    else x=x+LcdBigChr(x,y,*s++)+1; // Sens X
  }
}

/*---------------------------------------------------------------
  Nom          :  Init_Philips_Controler
  Description  :  Initialisations des registres du controleur Epson
                  Le signal CE_LCD est dja  "0"
  Arguments    :  Aucun
  Valeur renvoye : Aucune.
-----------------------------------------------------------------*/
void Init_Philips_Controler(void)
{
 P2OUT &= ~Lcd_RST_Pin; //Reset controleur (RST = "0")
 Delay(300); // Dlai de 0,5s environ
 P2OUT |= Lcd_RST_Pin; //Arrt reset (RST = "1")
 LcdSend_Cmd(SOFT_RESET);
 LcdSend_Cmd(SLEEP_OUT);
 LcdSend_Cmd(BOOSTER_ON);
 Delay(10);
 LcdSend_Cmd(DISPLAY_ON);
 LcdSend_Cmd(COLOR_INTERFACE);
 LcdSend_Data(COLOR_8_BIT);
 LcdSend_Cmd(COLOR_SET);
   //Les 8 niveaux de rouge cods sur 4 bits
   LcdSend_Data(0);LcdSend_Data(2);LcdSend_Data(4);LcdSend_Data(6);
   LcdSend_Data(9);LcdSend_Data(11);LcdSend_Data(13);LcdSend_Data(15);
   //Les 8 niveaux de vert cods sur 4 bits
   LcdSend_Data(0);LcdSend_Data(2);LcdSend_Data(4);LcdSend_Data(6);
   LcdSend_Data(9);LcdSend_Data(11);LcdSend_Data(13);LcdSend_Data(15);
   //Les 4 niveaux de bleu cods sur 4 bits
   LcdSend_Data(0);LcdSend_Data(4);LcdSend_Data(11);LcdSend_Data(15);
 // Paramtres pour cmd MEM_CONTROL Philips
 Sens_texte=0;
 Texte_MEM_CONTROL=MEM_VW|MEM_RGB|MEM_MX;//Sens X normal
 Image_MEM_CONTROL=MEM_RGB|MEM_MX;
   // MEM_RGB "0": ordre RGB, "1": BGR
   // MEM_LAO "0":haut vers bas, "1":bas vers haut
   // MEM_VW  "0":criture ds le sens X "1":criture ds le sens Y
   // MEM_MX  "0":normal, "1":miroir en X
   // MEM_MY  "0":normal, "1":miroir en Y
 LcdSend_Cmd(MEM_CONTROL);
   LcdSend_Data(Texte_MEM_CONTROL);
}

/*---------------------------------------------------------------
  Nom          :  Fill_Lcd
  Description  :  Remplir le LCD de la mme couleur
  Arguments    :  Coul_Lcd : code de la couleur
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void Fill_Lcd(char Couleur)
{
 int i;
 LcdSend_Cmd(SET_X_ADDR);
  LcdSend_Data(0);   //Start X address = 0
  LcdSend_Data(131); //End X address = 131
 LcdSend_Cmd(SET_Y_ADDR);
  LcdSend_Data(0);   //Start Y address = 0
  LcdSend_Data(131); //End Y address = 131
 LcdSend_Cmd(MEM_WRITE);
 for (i=0; i<132*132; i++) LcdSend_Data(Couleur);
}

/*--------------------------------------
  Nom          :  FillRectLcd
  Description  :  Remplir un rectangle sur le LCD avec la mme couleur
  Arguments    :  x1, y1 : coordonnes du 1 coin
                  x2, y2 : coordonnes du 2 coin
                  Couleur : code RVB sur 8 bits
  Valeur renvoye : aucune.
----------------------------------------*/
void FillRectLcd(char x1, char y1, char x2, char y2, char Couleur)
{
 int x,y;
 LcdSend_Cmd(SET_X_ADDR);
  LcdSend_Data(x1); //Start X address
  LcdSend_Data(x2); //End X address
 LcdSend_Cmd(SET_Y_ADDR);
  LcdSend_Data(y1); //Start Y address
  LcdSend_Data(y2); //End Y address
 LcdSend_Cmd(MEM_WRITE);
 for (x=x1; x<=x2; x++)
  for (y=y1; y<=y2; y++) LcdSend_Data(Couleur);
}

/*---------------------------------------------------------------
  Nom          :  Texte_0_degre
  Description  :  Parametrage du LCD pour un dessin de texte horizontal
  Arguments    :  aucun
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void Texte_0_degre(void)
{
 Sens_texte=0;
 Texte_MEM_CONTROL=MEM_MX|MEM_VW|MEM_RGB; //Sens X normal (Philips)
 LcdSend_Cmd(MEM_CONTROL); LcdSend_Data(Texte_MEM_CONTROL);
}

/*---------------------------------------------------------------
  Nom          :  Texte_90_degre
  Description  :  Parametrage du LCD pour un dessin de texte  90
  Arguments    :  aucun
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void Texte_90_degre(void)
{
 Sens_texte=1;
 Texte_MEM_CONTROL=MEM_RGB;//Sens Y normal
 LcdSend_Cmd(MEM_CONTROL); LcdSend_Data(Texte_MEM_CONTROL);
}

/*---------------------------------------------------------------
  Nom          :  Texte_180_degre
  Description  :  Parametrage du LCD pour un dessin de texte  180
  Arguments    :  aucun
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void Texte_180_degre(void)
{
 Sens_texte=2;
 Texte_MEM_CONTROL=MEM_MY|MEM_VW|MEM_RGB;//Sens X invers
 LcdSend_Cmd(MEM_CONTROL); LcdSend_Data(Texte_MEM_CONTROL);
}

/*---------------------------------------------------------------
  Nom          :  Texte_270_degre
  Description  :  Parametrage du LCD pour un dessin de texte  270
  Arguments    :  aucun
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void Texte_270_degre(void)
{
 Sens_texte=3;
 Texte_MEM_CONTROL=MEM_MX|MEM_MY|MEM_RGB; // Sens Y invers
 LcdSend_Cmd(MEM_CONTROL); LcdSend_Data(Texte_MEM_CONTROL);
}

/*---------------------------------------------------------------
  Nom          :  Affichage_Image
  Description  :  Affiche l'image monochrome Start_Image
  Arguments    :  aucun
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
__no_init char Image[2048] @ 0xF700;
void Affichage_Image(void)
{
 int i,x,y;
 char Couleur_Dessin, Couleur_Fond;
 LcdSend_Cmd(SET_X_ADDR);
  LcdSend_Data(2);   //Start X address
  LcdSend_Data(130); //End X address
 for (y=0; y<16; y++)
  {
   LcdSend_Cmd(SET_Y_ADDR);
    LcdSend_Data(2+y*8);   //Start Y address
    LcdSend_Data(2+y*8+7); //End Y address
   LcdSend_Cmd(MEM_WRITE);
   if (y<6) {Couleur_Dessin=Jaune; Couleur_Fond=Rouge;}
    else if (y<11) {Couleur_Dessin=Blanc; Couleur_Fond=Rouge;}
     else {Couleur_Dessin=7*32+4*4+3; Couleur_Fond=0*32+2*4+1;}
   for (x=0; x<128; x++)
    for (i=0; i<8; i++)
     {
       if (Image[x+y*128] & (1 << i)) LcdSend_Data(Couleur_Fond);
                                 else LcdSend_Data(Couleur_Dessin);
     }
  }
}

/*------------------------------------------------------------------------------------
                                        TIMER A
---------------------------------------------------------------------------------------*/

/*--------------------------------------------------
  Nom          :  InitTimerA_FSK
  Description  :  Initialisation du Timer A pour dmodulation FSK :
                    - TAR : continous up, horloge = SMCLK, pas d'int. TAIE
                    - CCR0 : Output Compare
                       - interruptions dclenches par EQU0 valides
                       - TACCR0 affect ds le programme d'int pour obtenir une priode
                         de EQU0 gale  Periode_IntTA0
                    - CCR1 : Output Compare
                       - OUT1 : Mode Reset/Set (Set avec EQU0 et Reset avec EQU1)
                       - TACCR1 affect ds le programme d'int pour obtenir une dure
                         de l'impulsion OUT1 gale  Pulse_OUT1
  Arguments    :  aucun
  Valeur renvoye : aucune.
----------------------------------------------------*/
void InitTimerA_FSK(void)
{
 TACTL=0x0210;  //Timer A : SMCLK, Up mode modulo TACCR0. Pas d'int TAIE
 TACCTL0=0x2010;//Output Compare avec interruption EQU0 valide
// TACCTL0=0x2000;//Output Compare avec interruption EQU0 inhibe
 TACCR0=Periode_IntTA0-1; //Priodes int TA0 = SMCLK/PeriodeIntTA0 = 6800Hz environ
 TACCTL1=0x2060;//Output Compare, OUT1 : Set/Reset
 TACCR1=Periode_IntTA0-1-Pulse_OUT1;//Pour produire une impulsion sur OUT1
                                    //de dure Pulse_OUT1
// P1SEL|=BIT2; //Test : P1.2/TA1=OUT1 pour controle  l'oscilloscope
// P1DIR|=BIT2;
}

/*--------------------------------------------------
  Nom          :  Calib_Int_TimerA0
  Note         :  Le coupleur SPI et le TimerA sont configurs
  Description  :  Calibration de la frquence des interruptions TimerA0 (6800Hz) :
                   - Rfrence : Xtal 32768Hz
                   - Coupleur SPI configur pour produire UCLK=ACLK/212
                   - Mesure de la priode avec voie 2 du Timer A
                   - Affectation TACCR0 avec la valeur adquate
                   - Cblage : sortie UCLK relie sur entre P1.3=CCI2A
  Arguments    :  aucun
  Valeur renvoye : aucune
----------------------------------------------------*/
void Calib_Int_TimerA0(void)
{
  char Old_BR1, Old_BR0, Old_U0TCTL;
  unsigned int Debut, Duree;
  div_t Nx;
  Old_BR1=U0BR1; Old_BR0=U0BR0; //Sauvegarde registre "Baud rate"
  Old_U0TCTL=U0TCTL; //Sauvegarde registre U0TCTL
  // Coupleur SPI
  U0BR1=0; U0BR0=212; //Registre U0BR=212
  U0TCTL=(U0TCTL & ~(SSEL1|SSEL0))|SSEL0; //Slection ACLK
  //Timer A
  P1SEL|=BIT3; //P1.3 dirig vers CCI2A
  TACCR0=0xFFFF; //TAR compte modulo 10000h
  TACCTL2=CM1|CAP;
              //CM="10" : capture aux flancs descendants
              //CCIS="00" : slection CCI2A=P1.3
              //SCS=0 : capture asynchrone
              //CAP=1 : mode capture
              //OUTMODE="000"
              //CCIE=0 : pas d'interruption
  TACTL|=TACLR; //Raz de TAR pour faciliter le calcul
  U0TXBUF=0xAA; //Pour dclencher 8 priodes d'horloge
  while ((TACCTL2 & CCIFG)==0) {};//Attendre le 1 flanc descendant
  Debut=TACCR2; //Mesure du "temps" du 1 flanc descendant
  TACCTL2&=~CCIFG;//Raz indicateur interruption
  while ((TACCTL2 & CCIFG)==0) {};//Attendre le 2 flanc descendant
  Duree=TACCR2-Debut; //Mesure de l'intervalle de temps entre les 2 flancs montants
  TACCTL2&=~CCIFG;//Raz indicateur interruption
  Nx=div(Duree,44);
  TACCR0=Nx.quot-1;//Frquence EQU0 = 6800Hz
  TACCR1=Nx.quot-1-Pulse_OUT1;//Pour produire une impulsion sur OUT1
                              //de dure Pulse_OUT1
  U0BR1=Old_BR1; U0BR0=Old_BR0; //Restitution registre "Baud rate"
  U0TCTL=Old_U0TCTL;//Restitution registre U0CTL
}

/*--------------------------------------------------
  Nom          :  InitTimerA_Delay
  Description  :  Initialisation du Timer A pour gnrer des dlais :
                    - TAR : continous up, horloge = ACLK/8, int. TAIE inhibe
                    - CCR0, CCR1 et CCR2 : non utiliss
  Arguments    :  aucun
  Valeur renvoye : aucune.
----------------------------------------------------*/
void InitTimerA_Delay(void)
{
 TACTL=TASSEL0|ID1|ID0|MC1;  //ACLK/8, mode continu, int TAIE inhibe
 TACCTL0=0 ;//Aucune fonction
 TACCTL1=0 ;//Aucune fonction
 TACCTL2=0 ;//Aucune fonction
}

/*------------------------------------------------------------------------------------
                                        CAN 10 bits
---------------------------------------------------------------------------------------*/

/*----------------------------------------------------
  Nom          :  InitCAN10
  Description  :  Initialisation du CAN 10 bits
                    - ADC10CLK = SMCLK
                    - SHI = TA1 (impulsion produite par le timer A)
                    - Mode : conversion dclenche d'une seule voie
                    - Entre analogique = A0 = P2.0
                    - Dure d'chantillonnage max = 64 ADC10CLK
                    - Rfrence : VCC
  Arguments    :  aucun
  Valeur renvoye : aucune.
------------------------------------------------------*/
void InitCAN10(void)
{
 ADC10CTL0=ADC10SR|REFOUT|REF2_5V|ADC10ON;
                                   //SREF="000" : VR+=VCC et VR-=VSS
                                   //ADC10SHT="00" : sample = 4 ADC10CLK
                                   //ADC10SR=1 : Fchantillonnage=6800Hz<50kHz
                                   //REFOUT=1 : Rfrence interne sortie
                                   //REFBURST=0 : VREF+ permanent
                                   //MSC=0 : conversions commandes par SHI
                                   //REF2_5V=1 : rfrence interne = 2,5V
                                   //REFON=0 : rfrence interne OFF (rduire consommation)
                                   //ADC10ON=1 : ADC10 oprationnel
                                   //ADC10IE=0 : pas d'interruption
                                   //ENC=0 : pour affecter les autres bits de config
 ADC10CTL1=INCH0|SHS0|ADC10DF|ADC10DIV2|ADC10DIV1|ADC10DIV0|ADC10SSEL1|ADC10SSEL0|CONSEQ1;
                                   //INCH="0001" : slection voie A1
                                   //SHS="01" : dclenchement avec OUT1 de Timer A
                                   //ADC10DF=1 : conversion en C2
                                   //ISSH=0 : pas d'inversion de OUT1
                                   //ADC10DIV=111 : division de SMCLK par 8
                                   //ADC10SEL="11" : horloge = SMCLK
                                   //CONSEQ="10" : conversion continue 1 voie
 ADC10AE=BIT1;   //Validation entre A1=P2.1
 ADC10CTL0|=ENC; //Activation du ADC10
}

/*------------------------------------------------------------------------------------
                                      UART
---------------------------------------------------------------------------------------*/
/*-------------------------------------------------
  Nom          :  InitUART
  Description  :  Initialisation du coupleur UART :
                   - 8 bits
                   - 1 bit STOP
                   - pas de bit de parit
  Arguments    :  Vitesse de transmission en bauds
  Valeur renvoye : aucune.
---------------------------------------------------*/
// Tableau de configuation pour une horloge ACLK de 32768Hz
const char Tab_BaudRate[4][2]={{0x1B,0x03}, //U0BR0=1Bh, U0MCTL=03h : 1200 bauds
                               {0x0D,0x6B}, //U0BR0=0Dh, U0MCTL=6Bh : 2400 bauds
                               {0x06,0x6F}, //U0BR0=06h, U0MCTL=6Fh : 4800 bauds
                               {0x0D,0x6B}};//U0BR0=03h, U0MCTL=4Ah : 9600 bauds
void InitUART(BaudRate Bauds)
{
  U0CTL=SWRST; //SWRST = "1" pour configurer l'USART en mode UART
  U0CTL=CHAR;  //PE=0 : pas de parit
               //PEV=0
               //SPB=0 : 1 bit Stop
               //CHAR=1 : 8 bits
               //LISTEN=0 : pas de loopback
               //SYNC=0 : mode UART
               //MM=0
  U0TCTL=(U0TCTL & TXEPT)|SSEL0;
               //CKPL=0 : horloge UCLK non inverse
               //SSEL="01" : BRCLK=ACLK
               //URXSE=0 : pas de fonction "Start-edge"
               //TXWAKE=0
  U0RCTL=0;    //Indicateurs FE,PE,OE, BRK et RXERR mis  0
               //URXEIE=0 : pas d'interruption si erreur en rception
               //URXWIE=0 : tous les caractres reus positionnent URXIFG0
               //RXWAKE=0
  U0BR0=Tab_BaudRate[Bauds][0]; //Baud rate
  U0BR1=0;                      //Octet de poids fort tjrs  0
  U0MCTL=Tab_BaudRate[Bauds][1];//Registre de modulation
  ME2|=URXE0;    //Validation rception UART dans TIM_A_Int
  U0CTL&=~SWRST; //SWRST = "0" fin de configuration dans TIM_A_Int
  P3SEL=BIT5;    //P3.5=URXD0
  IFG2=0; //Raz des indicateurs d'interruption,
          //notamment URXIFG0 et UTXIFG0
//  IE2=URXIE0; //Interruptions rception valides
}

/*-------------------------------------------------------------------------
  Nom          :  InitRS232
  Description  :  Initialisation du coupleur UART et des variables associes
  Arguments    :  Vitesse de transmission en bauds
  Valeur renvoye : aucune.
---------------------------------------------------------------------------*/
void InitRS232(BaudRate Bauds)
{
  int i;
  RXWriteIndex = RXReadIndex = RXCharCount = Lng_Message = Count_Bit = 0;
  for (i=0;i<RXBufSize;i++) RXBuffer[i]=0;
  // Controleur LCD non slectionn (CE_LCD = "1")
  P3OUT |= Lcd_CE_Pin ;
  InitUART(Bauds);//Initialisation UART : 8 bits, 1 bit Stop
                  //Interruptions en rception valides
}

/*---------------------------------------------------------------
  Nom         : ReadCharUART
  Description : Lecture du prochain caractre reu sur URXD0
  Arguments   : Aucun
  Retour      : Prochain caractre reu sur URXD s'il existe
                sinon : 0
-----------------------------------------------------------------*/
char ReadCharUART(void)
{
  char c;
  if (RXCharCount) // Un ou plusieurs caractres reus ?
   {
     c = RXBuffer[RXReadIndex++]; // Lecture du prochain caractre
                                  // et incrmentation de l'index lecture
     RXReadIndex &= RXBufSize-1; // Modulo RXBufSize
     RXCharCount--;  // Dcrmentation compteur d'octets
     return(c);
   }
  else return(0); // if there is no new char
}

/*----------------------------------------------------
  Nom          :  Init_ZN426E
  Description  :  Initialisation du MSP430 pour utiliser le ZN426E
  Arguments    :  aucun
  Valeur renvoye : aucune.
------------------------------------------------------*/
void Init_ZN426E(void)
{
 P3DIR=0xFF; //Sortie vers ZN426E
 P3SEL=0   ; //Port 3 en sortie CPU
}

/*----------------------------------------------------
  Nom          :  Init_Var
  Description  :  Initialisation des variables
  Arguments    :  aucun
  Valeur renvoye : aucune.
------------------------------------------------------*/
void Init_Var(void)
{
 int i;
 for (i=0;i<5;i++) WDF_PARMS[i]=0;
 end_of_parms=0;
 RXWriteIndex = RXReadIndex = RXCharCount = Lng_Message = Count_Bit = 0;
 for (i=0;i<RXBufSize;i++) RXBuffer[i]=0;
 R6_T=0;
 Compt_Int_6800=0; // Raz du compteur d'interruptions
}

/*------------------------------------------------------------------------------------
                                       Programme principal
--------------------------------------------------------------------------------------*/
void main(void)
{
 WDTCTL=WDTPW+WDTHOLD;       // Arrt du chien de garde
 // Horloge basse
 SetDCOClock(3,1); // FDCO = 190kHz typique
 // Initialisations ports E/S
 P1DIR=TEST1|TEST2|TEST3;  // Port 1
 P2DIR=Lcd_RST_Pin|PWR_ON; // Port 2
 P2DIR|=RXD; //Sur plaquette connection sans soudure
 P3DIR=Lcd_CE_Pin|Lcd_DIN_Pin|Lcd_CLK_Pin|Lcd_DC_Pin|RXD|CS_CNA|LDAC|TEST4;//Port 3
 P3OUT=Lcd_CE_Pin ; // Controleur non slectionn (CE = "1")
 P2OUT=Lcd_RST_Pin; // et pas de reset (RST = "1"). Aops non aliments
 // Dtection sonnerie
 P2IE |=SONNE; //Validation interruption sur P2.5
 P2IES|=SONNE; //Sur flanc descendant
 // Initialisation USART en mode SPI (pour affichage initial)
 InitSPI_Lcd();//Initialisation du coupleur SPI pour Lcd
 Init_Philips_Controler(); //Initialisation controleur Lcd
 Affichage_Image();        // Ecran initial
 InitTimerA_Delay(); //Initialisation TimerA pour raliser monostable (int inhibe)
 InitCAN10();//Initialisation CAN 10 bits (mais rfrence interne OFF pour consommation)
 IND_R4=0;
 Compt_Sonnerie=0; // Raz du compteur de priodes de sonnerie
 Premier_Numero=0;
 // Validation interruptions ds CPU
 _EINT();
/*--------------------
   Boucle principale
 ---------------------*/
 do
  {
   /* Arrt CPU hors interruptions
   Low power mode 3 (LPM3); SCG1=1, SCG0=1, OscOff=0, CPUOff=1 */
   if (IND_R4==0) __low_power_mode_3();//Tout  l'arrt, sauf ACLK
   if (IND_R4 & NEW_MSG)
   {
    // Passer en mode "basse consommation" et affichage du numro appelant
    _DINT();
    SetDCOClock(3,1);  // Horloge basse : FDCO = 190kHz typique
    P2OUT&=~PWR_ON;    // Arrt alimentation Aop
    ADC10CTL0&=~REFON; // Arrt rfrence pour rduire la consommation
    InitTimerA_Delay();// Initialisation TimerA pour raliser monostable (int inhibe)
    InitSPI_Lcd();     // Initialisation du coupleur SPI pour Lcd
    if (!(IND_R4 & TIME_OUT))
     {
      if (!Premier_Numero)
       {
         Fill_Lcd(Bleu);  // Effacer l'cran en bleu
         Couleur_texte=7*32+7*4+3; Fond_texte=0*32+0*4+3;
         LcdLittleStr(2,2,"Derniers numeros appelant:");
         Fond_texte=0*32+3*4+0; Couleur_texte=7*32+7*4+1;
         Pos_Y_Msg=111;   // Position Y initiale des messages
         Premier_Numero=1;
       }
      RXBuffer[Lng_Message-1]=0;
      FillRectLcd(2,Pos_Y_Msg+4,5,Pos_Y_Msg+8,Bleu);
      Pos_Y_Msg+=16; // Position suivante : 16 pixels en-dessous
      if (Pos_Y_Msg > 114) Pos_Y_Msg=15;
      FillRectLcd(2,Pos_Y_Msg+4,5,Pos_Y_Msg+8,Rouge);
      FillRectLcd(7,Pos_Y_Msg,130,Pos_Y_Msg+15,Bleu);
      LcdBigStr(7,Pos_Y_Msg,RXBuffer+4);
     }
    IND_R4=0;
    Compt_Sonnerie=0; // Raz du compteur de priodes de sonnerie
    _EINT();
   }
   P1OUT^=TEST3; //Test provisoire
  }
 while (1);
}

/*------------------------------------------------------------------------------------
                                     Programmes d'interruption
--------------------------------------------------------------------------------------*/
/*-----------------------
   Interruption Port P2
 ------------------------
 Active  chaque flanc descendant sur P2.5=SONNE :
   - Prochaine interruption TAI ds 500mS (monostable redclenchable)
     (dure mini phase B de la transmission) */
 #pragma vector=PORT2_VECTOR
 __interrupt void Int_PA2()
 {
  Compt_Sonnerie++;// Incrmentation du compteur de priodes de sonnerie
  TAR=0xFFFF-2047; // Prochaine interruption TAI ds 500mS
  TACTL&=~TAIFG;   // Raz indicateur TAIFG
  TACTL|=TAIE;     // Validation interruption TAI
  P2IFG&=~SONNE;   // Acquitement interruption
  __low_power_mode_off_on_exit();
 }

/*-------------------
   Interruption TAI
 --------------------
 Active 500mS aprs arrt sonnerie puis inhibe
  - Activer la dmodulation FSK par interruptions TimerA au retour d'int. */
#pragma vector=TIMERA1_VECTOR
__interrupt void Int_TAI()
{
 if (Compt_Sonnerie>10) // Nb de priodes de sonnerie suffisants
  {
   // Alimentation Aop
   P2OUT|=PWR_ON;
   // Horloge haute
   SetDCOClock(3,6); // FDCO = 1,9MHz typique
   ADC10CTL0|=REFON; //Activer la rfrence interne
   // Initialisation et calibration TimerA pour dmodulation FSK
   InitSPI_Lcd();//Initialisation du coupleur SPI pour Lcd
   InitTimerA_FSK(); //Initialisation du Timer A (interruptions 6800Hz)
   Calib_Int_TimerA0(); //Calibration frquence d'chantillonnage (6800Hz)
   // Init_ZN426E(); //Port 3 sur ZN426E
   // Initialisation USART en mode UART (pour rception asynchrone)
   InitRS232(B1200);//Initialisation UART : 1200 bauds, 8 bits, 1 bit Stop
                    //Interruptions en rception non valides
   // Initialisations variables pour acquisition trame CLIP
   Init_Var();      //Initialisation des variables pour dmod FSK
   TACTL&=~TAIE;    //Inhibition prochaines interruptions
   IND_R4=WAIT_SMMR;//Phase d'attente du signal SMMR
  }
 TACTL&=~TAIFG;     //Raz indicateur TAIFG (acquitement interruption)
 __low_power_mode_off_on_exit();
}
