/****************************************************
* Dmodulation CLIP avec MSP430F1132                *
****************************************************/

/* Le programme d'interruption Timer A (OC 0) est dcrit dans le module
   TA0_int_CLIP.s43.
   Il comporte aussi la dclaration de son vecteur d'interruption */

/*------------------------------------------------------------------------------------
  Nom          :  Demod_CLIP_1232_Nokia_3310.c
  Description  :  Dmodulation CLIP
                  Pour carte CLIP de CREMMEL M. avec LCD Nokia 3310
  Author       :  05/05/2006 - CREMMEL Marcel
  History      :  12/04/2006 - Premire version
--------------------------------------------------------------------------------------*/
#include <msp430x12x2.h>
#include <stdbool.h> // Pour le type "bool"
#include <stdlib.h>  // Pour le type div_t

__regvar __no_init int IND_R4 @ __R4; // Rservation registre R4
/*------------------------------
* Indicateurs du registre IND_R4
--------------------------------*/
#define WAIT_SMMR        1  // Indique la phase d'attente du signal SMMR
#define WAIT_MARK        2  // Indique la phase d'attente du signal MARK
#define WAIT_MSG         4  // Indique la phase de rception du message
#define NEW_MSG          8  // Indique un nouveau message
#define TIME_OUT         16 // Indique un problme pdt la rception de la trame
#define LAST_RXD         32 // Etat prcdent de RXD

/*------------------------------------------------------------------------------------
                                         Constantes
--------------------------------------------------------------------------------------*/

/*------------------
       Cblage
--------------------*/
// Ports SPI
#define DIN_SPI     BIT1  //  P3.1 = SOMI = SDIN
#define CLK_SPI     BIT3  //  P3.3 = UCLK = SCLK

// Connexions LCD Nokia (transmission via SPI)
#define Lcd_CE_Pin  BIT0  //  P3.0 = CE_LCD
#define Lcd_DC_Pin  BIT2  //  P3.2 = D/C
#define Lcd_RST_Pin BIT2  //  P2.2 = RESET_LCD

// Connexions CNA MAX550BCPA (transmission via SPI)
#define CS_CNA   BIT6   // P3.6 = CS_CNA, actif  "0"
#define LDAC     BIT4   // P3.4 = LDAC, actif  "0"

// Sortie RXD dmodule
#define RXD      BIT7   // P3.7 = sortie dmodule RXD

// Autre signaux
#define PWR_ON   BIT0   // P2.0 = PWR_ON (alim Aops)
#define SONNE    BIT5   // P2.5 = SONNE (entre dtection de sonnerie)

// Signaux de test
#define TA1      BIT2   // P1.2 = signal de test TA1

/*-------------------------
  Constantes LCD Nokia 3310
---------------------------*/
#define Lcd_X_res       84
#define Lcd_Y_res       48
// Commandes controleur PCD8544
#define Function_Set    0x20    //Bit 2 = PD, Bit 1 = V, Bit 1 = H
#define Lcd_ON_Norm_H   0x20    //Lcd actif, adressage H, mode normal
#define Lcd_ON_Norm_V   0x22    //Lcd actif, adressage V, mode normal
#define Lcd_ON_Etendu_H 0x21    //Lcd actif, adressage H, mode tendu (H=1)
#define Lcd_OFF_Norm_H  0x00    //Lcd "power down", adressage H, mode normal
// Commandes normales (bit H="0")
#define Set_Config      0x08    //Bit 2 = D, bit 0 = E
#define Set_Adr_Y       0x40    //Adresse Y dans 3 bits LSB (0  5)
#define Set_Adr_X       0x80    //Adresse X dans 7 bits LSB (0  83)
// Commandes tendues (bit H="1")
#define Set_Bias        0x10    //Bias Mode dans bits 2  0
#define Set_Vop         0x80    //Vop dans bits 6  0
#define Set_TC          0x04    //Coef de temprature dans bits 1  0
// Positions des paramtres sur le LCD
#define Pos_FRX_X       5       //Position X de la frquence de rception (0  83)
#define Pos_FRX_Y       2       //Position Y de la frquence de rception (0  5)
#define Pos_Filtre_X    41      //Position X de la frquence de rception (0  83)
#define Pos_Filtre_Y    4       //Position Y de la frquence de rception (0  5)

/*------------------
  Constantes Timer A
--------------------*/
// Interruption Timer A, voie 0
#define Pulse_OUT1      100     //Dure de l'impulsion de dclenchement du CAN
#define Periode_IntTA0  588     //Soit une frquence de 6800Hz avec horloge de 4MHZ

/*---------------
  Constantes UART
-----------------*/
#define RXBufSize 32 // Taille du buffer de rception (puissance de 2)

/*------------------------------------------------------------------------------------
                                    Dfinition de type
--------------------------------------------------------------------------------------*/
typedef enum
{
    Lcd_Cmd  = 0,
    Lcd_Data = 1
} LcdCmdData; // Type de transfert : commande ou donnes

typedef enum
{
    Blanc = 0,
    Noir  = 1
} Couleur; // Type de couleur

typedef enum
{
    B1200 = 0,
    B2400 = 1,
    B4800 = 2,
    B9600 = 3
} BaudRate; // Vitesse de transmission UART

/*------------------------------------------------------------------------------------
                                    Variables globales
-------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------
   Variables du programme d'interruption "TA0_int_CLIP.s43"
-------------------------------------------------------------
-----------------
   Filtre passe-bas
-----------------*/
int WDF_PARMS[5];  // Pile du filtre passe-bas
int Count_Bit;    // Compteur d'chantillons  "1" pour dtecter dbut trame
int R6_T;         // Fonction "retard"
/*-------------------------
  Compteurs d'interruptions
---------------------------*/
int Compt_Int_6800; // Permet d'arrter la procdure de rception de la trame CLIP
                    // en cas de problme (pas de trame, perturbations diverses, ...)
int Compt_Sonnerie; // Pour dclencher la procdure de rception de la trame CLIP
                    // qu'aprs un signal de sonnerie
/*-----------------
       UART
-------------------*/
// Index en rception :
int RXReadIndex, RXWriteIndex;
// Comptage des octets reus :
char RXCharCount;
char Lng_Message; // Pour mmoriser la longueur des messages
// Buffer de rception
char RXBuffer[RXBufSize];

/*-----------------------------------------------------------
                     Autres variables
-------------------------------------------------------------*/
/*-----------------
        LCD
-------------------*/
char Adr_X ; //Copie logicielle de l'adresse X du PCD8544 (0  83)
char Adr_Y ; //Copie logicielle de l'adresse Y du PCD8544 (0  5)

char Pos_Y_Msg;      //Position Y des messages reus
char Premier_Numero; //Indique la rception du 1 numro

/*------------------------------------------------------------------------------------
                                 Constantes en mmoire flash
-------------------------------------------------------------------------------------*/
/* ---------------------
   Polices de caractres
   ---------------------
Gnrateur de caractre en mmoire flash
Code Ascii, format 5x7 pixels.
Format : Y    X   1   2   3   4   5
         1       D10 D20 D30 D40 D50
         2       D11 D21 D31 D41 D51
         3       D12 D22 D32 D42 D52
         4       D13 D23 D33 D43 D53
         5       D14 D24 D34 D44 D54
         6       D15 D25 D35 D45 D55
         7       D16 D26 D36 D46 D56
1 octet : D16-D15-D14-D13-D12-D11-D10, bit 7 toujours  "0"
2 octet : D26-D25-D24-D23-D22-D21-D20, bit 7 toujours  "0"
3 octet : D36-D35-D34-D33-D32-D31-D30, bit 7 toujours  "0"
4 octet : D46-D45-D44-D43-D42-D41-D40, bit 7 toujours  "0"
5 octet : D56-D55-D54-D53-D52-D51-D50, bit 7 toujours  "0"
--------------------------------------------------------------------------------------*/
const char FontLookup[][5] =
{
    { 0x00, 0x00, 0x00, 0x00, 0x00 },   // sp
    { 0x00, 0x00, 0x2f, 0x00, 0x00 },   // !
    { 0x00, 0x07, 0x00, 0x07, 0x00 },   // "
    { 0x14, 0x7f, 0x14, 0x7f, 0x14 },   // #
    { 0x24, 0x2a, 0x7f, 0x2a, 0x12 },   // $
    { 0xc4, 0xc8, 0x10, 0x26, 0x46 },   // %
    { 0x36, 0x49, 0x55, 0x22, 0x50 },   // &
    { 0x00, 0x05, 0x03, 0x00, 0x00 },   // '
    { 0x00, 0x1c, 0x22, 0x41, 0x00 },   // (
    { 0x00, 0x41, 0x22, 0x1c, 0x00 },   // )
    { 0x14, 0x08, 0x3E, 0x08, 0x14 },   // *
    { 0x08, 0x08, 0x3E, 0x08, 0x08 },   // +
    { 0x00, 0x00, 0x50, 0x30, 0x00 },   // ,
    { 0x10, 0x10, 0x10, 0x10, 0x10 },   // -
    { 0x00, 0x60, 0x60, 0x00, 0x00 },   // .
    { 0x20, 0x10, 0x08, 0x04, 0x02 },   // /
    { 0x3E, 0x51, 0x49, 0x45, 0x3E },   // 0
    { 0x00, 0x42, 0x7F, 0x40, 0x00 },   // 1
    { 0x42, 0x61, 0x51, 0x49, 0x46 },   // 2
    { 0x21, 0x41, 0x45, 0x4B, 0x31 },   // 3
    { 0x18, 0x14, 0x12, 0x7F, 0x10 },   // 4
    { 0x27, 0x45, 0x45, 0x45, 0x39 },   // 5
    { 0x3C, 0x4A, 0x49, 0x49, 0x30 },   // 6
    { 0x01, 0x71, 0x09, 0x05, 0x03 },   // 7
    { 0x36, 0x49, 0x49, 0x49, 0x36 },   // 8
    { 0x06, 0x49, 0x49, 0x29, 0x1E },   // 9
    { 0x00, 0x36, 0x36, 0x00, 0x00 },   // :
    { 0x00, 0x56, 0x36, 0x00, 0x00 },   // ;
    { 0x08, 0x14, 0x22, 0x41, 0x00 },   // <
    { 0x14, 0x14, 0x14, 0x14, 0x14 },   // =
    { 0x00, 0x41, 0x22, 0x14, 0x08 },   // >
    { 0x02, 0x01, 0x51, 0x09, 0x06 },   // ?
    { 0x32, 0x49, 0x59, 0x51, 0x3E },   // @
    { 0x7E, 0x11, 0x11, 0x11, 0x7E },   // A
    { 0x7F, 0x49, 0x49, 0x49, 0x36 },   // B
    { 0x3E, 0x41, 0x41, 0x41, 0x22 },   // C
    { 0x7F, 0x41, 0x41, 0x22, 0x1C },   // D
    { 0x7F, 0x49, 0x49, 0x49, 0x41 },   // E
    { 0x7F, 0x09, 0x09, 0x09, 0x01 },   // F
    { 0x3E, 0x41, 0x49, 0x49, 0x7A },   // G
    { 0x7F, 0x08, 0x08, 0x08, 0x7F },   // H
    { 0x00, 0x41, 0x7F, 0x41, 0x00 },   // I
    { 0x20, 0x40, 0x41, 0x3F, 0x01 },   // J
    { 0x7F, 0x08, 0x14, 0x22, 0x41 },   // K
    { 0x7F, 0x40, 0x40, 0x40, 0x40 },   // L
    { 0x7F, 0x02, 0x0C, 0x02, 0x7F },   // M
    { 0x7F, 0x04, 0x08, 0x10, 0x7F },   // N
    { 0x3E, 0x41, 0x41, 0x41, 0x3E },   // O
    { 0x7F, 0x09, 0x09, 0x09, 0x06 },   // P
    { 0x3E, 0x41, 0x51, 0x21, 0x5E },   // Q
    { 0x7F, 0x09, 0x19, 0x29, 0x46 },   // R
    { 0x46, 0x49, 0x49, 0x49, 0x31 },   // S
    { 0x01, 0x01, 0x7F, 0x01, 0x01 },   // T
    { 0x3F, 0x40, 0x40, 0x40, 0x3F },   // U
    { 0x1F, 0x20, 0x40, 0x20, 0x1F },   // V
    { 0x3F, 0x40, 0x38, 0x40, 0x3F },   // W
    { 0x63, 0x14, 0x08, 0x14, 0x63 },   // X
    { 0x07, 0x08, 0x70, 0x08, 0x07 },   // Y
    { 0x61, 0x51, 0x49, 0x45, 0x43 },   // Z
    { 0x00, 0x7F, 0x41, 0x41, 0x00 },   // [
    { 0x55, 0x2A, 0x55, 0x2A, 0x55 },   // 55
    { 0x00, 0x41, 0x41, 0x7F, 0x00 },   // ]
    { 0x04, 0x02, 0x01, 0x02, 0x04 },   // ^
    { 0x40, 0x40, 0x40, 0x40, 0x40 },   // _
    { 0x00, 0x01, 0x02, 0x04, 0x00 },   // '
    { 0x20, 0x54, 0x54, 0x54, 0x78 },   // a
    { 0x7F, 0x48, 0x44, 0x44, 0x38 },   // b
    { 0x38, 0x44, 0x44, 0x44, 0x20 },   // c
    { 0x38, 0x44, 0x44, 0x48, 0x7F },   // d
    { 0x38, 0x54, 0x54, 0x54, 0x18 },   // e
    { 0x08, 0x7E, 0x09, 0x01, 0x02 },   // f
    { 0x0C, 0x52, 0x52, 0x52, 0x3E },   // g
    { 0x7F, 0x08, 0x04, 0x04, 0x78 },   // h
    { 0x00, 0x44, 0x7D, 0x40, 0x00 },   // i
    { 0x20, 0x40, 0x44, 0x3D, 0x00 },   // j
    { 0x7F, 0x10, 0x28, 0x44, 0x00 },   // k
    { 0x00, 0x41, 0x7F, 0x40, 0x00 },   // l
    { 0x7C, 0x04, 0x18, 0x04, 0x78 },   // m
    { 0x7C, 0x08, 0x04, 0x04, 0x78 },   // n
    { 0x38, 0x44, 0x44, 0x44, 0x38 },   // o
    { 0x7C, 0x14, 0x14, 0x14, 0x08 },   // p
    { 0x08, 0x14, 0x14, 0x18, 0x7C },   // q
    { 0x7C, 0x08, 0x04, 0x04, 0x08 },   // r
    { 0x48, 0x54, 0x54, 0x54, 0x20 },   // s
    { 0x04, 0x3F, 0x44, 0x40, 0x20 },   // t
    { 0x3C, 0x40, 0x40, 0x20, 0x7C },   // u
    { 0x1C, 0x20, 0x40, 0x20, 0x1C },   // v
    { 0x3C, 0x40, 0x30, 0x40, 0x3C },   // w
    { 0x44, 0x28, 0x10, 0x28, 0x44 },   // x
    { 0x0C, 0x50, 0x50, 0x50, 0x3C },   // y
    { 0x44, 0x64, 0x54, 0x4C, 0x44 }    // z
};

/*--------------------------------------
Gnrateur de caractres : chiffres 0  9
Nombre de caractres : 10
Rsolution X : 10
Rsolution Y : 14 */
const char MyFontTab[][10] =
{
    { 252, 254,   3,   3,   3,   3,   3,   3, 254, 252  }, // 0 octet LSB
    {   0,  16,  24,  12,   6, 255, 255,   0,   0,   0  }, // 1 octet LSB
    {   2,   3, 131, 131, 131, 131, 131, 195, 254, 124  }, // 2 octet LSB
    {   2,   3,   3, 195, 195, 195, 195, 199, 254,  60  }, // 3 octet LSB
    { 192, 224,  48,  24,  12, 198, 195,   1,   0,   0  }, // 4 octet LSB
    {  31,  63,  99,  99,  99,  99,  99,  99, 195, 131  }, // 5 octet LSB
    { 252, 254, 195, 195, 195, 195, 195, 195, 131,   0  }, // 6 octet LSB
    {   2,   3,   3,   3, 131, 195,  99,  51,  31,  15  }, // 7 octet LSB
    { 124, 254, 195, 195, 195, 195, 195, 195, 254,  60  }, // 8 octet LSB
    { 124, 254, 195, 195, 195, 195, 195, 195, 254, 252  }, // 9 octet LSB
    {  15,  31,  48,  48,  48,  48,  48,  48,  31,  15  }, // 0 octet MSB
    {   0,   0,   0,  48,  48,  63,  63,  48,  48,   0  }, // 1 octet MSB
    {  62,  63,  51,  49,  49,  49,  49,  49,  48,  48  }, // 2 octet MSB
    {  16,  48,  48,  48,  48,  48,  48,  56,  31,  15  }, // 3 octet MSB
    {   3,   3,   3,   3,   3,  63,  63,   3,   3,   3  }, // 4 octet MSB
    {  16,  48,  48,  48,  48,  48,  48,  48,  31,  15  }, // 5 octet MSB
    {  15,  31,  48,  48,  48,  48,  48,  48,  31,  15  }, // 6 octet MSB
    {  56,  60,   6,   3,   1,   0,   0,   0,   0,   0  }, // 7 octet MSB
    {  15,  31,  48,  48,  48,  48,  48,  48,  31,  15  }, // 8 octet MSB
    {   0,  48,  48,  48,  48,  48,  48,  48,  31,  15  }  // 9 octet MSB
};

/*--------------------------------------
          Image d'accueil
  --------------------------------------*/
const char Image_Accueil[] =
{
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0, 128, 224, 240, 248,  56,  28,  28,  28,  28,
  28,  60,  56, 120, 112,  32,   0,   0,   0, 252, 252, 252,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
 252, 252, 252,   0,   0,   0,   0, 252, 252, 252,  28,  28,
  28,  28,  28,  28,  60, 248, 248, 224,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,  63, 255, 255, 224, 128,   0,   0,   0,   0,
   0, 128, 128, 192, 192, 128,   0,   0,   0, 255, 255, 255,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
 255, 255, 255,   0,   0,   0,   0, 255, 255, 255,  28,  28,
  28,  28,  28,  28,  30,  15,   7,   3,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   1,   3,   3,   7,   7,   7,   7,
   7,   7,   3,   3,   1,   0,   0,   0,   0,   7,   7,   7,
   7,   7,   7,   7,   7,   7,   7,   7,   7,   0,   0,   0,
   7,   7,   7,   0,   0,   0,   0,   7,   7,   7,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0, 128, 112,  46,  33,  46, 112, 132, 255,   5,   4, 255,
   5,   0, 253,   0, 248,   4,   4, 136,   0, 255,   8,   4,
   4, 248,   0, 200,  36,  36,  36, 248,   0, 248,   4,   4,
 136, 252,   0, 248,  36,  36,  36, 184,   0,   0,   0,   0,
 248,   4,   4, 136, 255,   0, 252,   0,   0, 128, 252,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0, 192, 128,  64,  64, 128,   0, 192,   0,   0,   0,
 192,   1, 192, 128,  64, 128, 128,  65, 129,   0,   0, 129,
  64,  80,  73, 128,   0, 193, 129,  64,   0, 129,  64,  64,
  64, 129,   0,   0,   1,   1, 129,  65,  64,  68, 137,   9,
 200, 135,  64,  64, 129,   1, 193, 128,  64,  64, 128,   0,
 128,  65,  65,  64, 129,   0, 240,   1, 129,  64,  65,  64,
 128,   0, 192, 128,  64,  64, 128,  64, 240,  64,   0,   0,
   0,   0,  31,   0,   0,   0,  31,   0,  15,  16,  16,   8,
  31,   0,  31,   0,   0,  31,   0,   0,  31,   0,   0,  15,
  18,  18,  18,  11,   0,  31,   0,   0,   0,  15,  16,  16,
  16,  15,   0,   0,   0,   0,  12,  18,  18,  18,  31,   0,
 255,   8,  16,  16,  15,   0, 255,   8,  16,  16,  15,   0,
  15,  18,  18,  18,  11,   0,  31,   0,  12,  18,  18,  18,
  31,   0,  31,   0,   0,   0,  31,   0,  31,  16,   0,   0
};

/*------------------------------------------------------------------------------------
                                        HORLOGES
---------------------------------------------------------------------------------------*/

/*--------------------------------------------------
  Nom          :  SetDCOClock
  Description  :  Affectation frquence DCO
  Arguments    :  DCO : valeur des 3 bits DCO
                  RES : valeur des 3 bits RESEL
  Valeur renvoye : aucune.
----------------------------------------------------*/
void SetDCOClock(int DCO,int RES)
{
  DCOCTL=(DCOCTL & ~(DCO2|DCO1|DCO0)) | (DCO << 5); //Affectation DCO[2..0]
  BCSCTL1=(BCSCTL1 & ~(RSEL2|RSEL1|RSEL0)) | RES  ; //Affectation RESEL[2..0]
}

/*------------------------------------------------------------------------------------
                                   Fonctions de gestion du LCD
--------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------
  Nom          :  InitSPI_Lcd
  Description  :  Initialisation du coupleur SPI
                  Il est utilis pour la transmission vers le LCD
  Arguments    :  aucun
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void InitSPI_Lcd(void)
{
  U0CTL=SWRST; //SWRST = "1" pour configurer l'USART en mode SPI
  U0CTL=SWRST|CHAR|SYNC|MM; //Mode Master SPI 8 bits
  U0TCTL=CKPH|SSEL1|STC; //CKPH/CKPL=10 : format PCD8544
                         //SSEL=10 : SMCLK
                         //STC=1 : STE inhib
  U0BR0=4; //
  U0BR1=0; // U0BR = 4 : Baud rate = SMCLK/4
  U0MCTL=0; //Impos en mode SPI
  ME2|=USPIE0; //Validation SPI
  U0CTL&=~SWRST; //SWRST = "0" fin de configuration
  P3SEL|=DIN_SPI|CLK_SPI;
}

/*---------------------------------------------------------------
  Nom          :  LcdSend
  Description  :  Envoi d'un octet vers le controleur du LCD via coupleur SPI
  Arguments    :  data -> Donne ou commande  transmettre
                  cd   -> Type de transfert : commande (Lcd_Cmd) ou donne (Lcd_Data)
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void LcdSend(char Data, LcdCmdData CD)
{
 // Attendre fin de transmission
 while ((U0TCTL & TXEPT)==0) {};
 // Affectation signal D/C
 if (CD == Lcd_Data) P3OUT |=  Lcd_DC_Pin;
                else P3OUT &= ~Lcd_DC_Pin;
 // Transmission srie de l'octet
 U0TXBUF=Data;
}

/*---------------------------------------------------------------
  Nom          :  Delay
  Description  :  Production d'un dlai logiciel
  Arguments    :  aucun
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void Delay(int Delai)
{
  int i;
  for ( i = -Delai; i < Delai; i++ );
}

/*---------------------------------------------------------------
  Nom          :  LcdInit
  Description  :  Initialisation du PCD8544
  Arguments    :  aucun
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
// Note : la commande de contraste est sans effet
void LcdInit(void)
{
  P3OUT |= Lcd_CE_Pin ; // Controleur non slectionn (CE = "1")
  P2OUT |= Lcd_RST_Pin; // et pas de reset (RST = "1")
  Delay(3000); // Dlai de xx5s environ
  P2OUT &= ~Lcd_RST_Pin; //Reset PCD8544 (RST = "0")
  Delay(3000); // Dlai de xxs environ
  P2OUT |= Lcd_RST_Pin; //Arrt reset PCD8544 (RST = "1")
  Adr_X=0; Adr_Y=0;
  // Slection du controleur (actif  "0").
  P3OUT &= ~Lcd_CE_Pin; // CE = "0"
  LcdSend(Lcd_ON_Etendu_H, Lcd_Cmd); //Mode commandes tendues
  // LcdSend(Set_Vop+72,Lcd_Cmd); //Set LCD Vop (contrast)
  // LcdSend(Set_TC+2,Lcd_Cmd);   //Set temp. coefficient
  LcdSend(Set_Bias+3,Lcd_Cmd);    //LCD Bias mode 1:48
  LcdSend(Lcd_ON_Norm_H,Lcd_Cmd); //LCD standard commands. Horizontal addressing mode.
  LcdSend(Set_Config+4,Lcd_Cmd);  //LCD in normal mode (DE = 10)
}

/*---------------------------------------------------------------
  Nom          :  LcdGotoXY
  Description  :  Affecter les adresses X et Y du PCD8544
  Arguments    :  X : position H (0  83)
                  Y : position V (0  5)
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void LcdGotoXY(char x, char y)
{
  Adr_X=x; Adr_Y=y;
  LcdSend(Set_Adr_X+Adr_X,Lcd_Cmd); //Affectation adresse X
  LcdSend(Set_Adr_Y+Adr_Y,Lcd_Cmd); //Affectation adresse Y
}

/*---------------------------------------------------------------
  Nom          :  Fill_Lcd
  Description  :  Remplir le LCD de la mme couleur
  Arguments    :  Coul_Lcd : Noir ou Blanc
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void Fill_Lcd(Couleur Coul_Lcd)
{
 int x,y;
 char b;
 if (Coul_Lcd==Noir) b=255;
                else b=0;
 for (y=0; y<Lcd_Y_res/8+1; y++)
  {
   LcdGotoXY(0,y);
   for (x=0; x<Lcd_X_res; x++) LcdSend(b,Lcd_Data);
  }
 LcdGotoXY(0,0);
}

/*---------------------------------------------------------------
  Nom          :  LcdChar
  Description  :  Dessin d'un caractre  la position actuelle des compteurs X et Y
  Arguments    :  Code Ascii du caractre
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void LcdChar(char c)
{
 int i;
 for (i=0; i<5; i++) LcdSend(FontLookup[c-32][i],Lcd_Data);
 Adr_X += 6;
}

/*---------------------------------------------------------------
  Nom          :  LcdString
  Description  :  Dessin d'une phrase  la position actuelle des compteurs X et Y
  Arguments    :  Pointeur de la chane de caractres
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void LcdString(char X, char Y, char *Phrase)
{
 LcdGotoXY(X,Y);
 while (*Phrase)
  {
   LcdChar(*Phrase++);
   LcdSend(0,Lcd_Data); // Espace inter-caractres
   Adr_X++;
  }
}

/*---------------------------------------------------------------
  Nom          :  LcdBigDigit
  Description  :  Dessin d'un chiffre 14x10 pixels  la position actuelle des
                  compteurs X et Y
  Arguments    :  Chiffre cod BCD
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void LcdBigDigit(char b)
{
 int i;
 for (i=0; i<10; i++) LcdSend(MyFontTab[b][i],Lcd_Data); // D'abord les octets LSB
 LcdSend(Set_Adr_X+Adr_X,Lcd_Cmd);   // Retour au dbut du dessin
 LcdSend(Set_Adr_Y+Adr_Y+1,Lcd_Cmd); // Octets MSB  la ligne de 8 pixels infrieure
 for (i=0; i<10; i++) LcdSend(MyFontTab[b+10][i],Lcd_Data); // Puis les octets MSB
 Adr_X += 12;
 LcdSend(Set_Adr_Y+Adr_Y,Lcd_Cmd); //Prochain chiffre sur la mme ligne
 LcdSend(Set_Adr_X+Adr_X,Lcd_Cmd); //et 12 pixels  droite
}

/*---------------------------------------------------------------
  Nom          :  LcdBigString
  Description  :  Dessin d'une suite de chiffres  la position actuelle
                  des compteurs X et Y
  Arguments    :  Pointeur de la chane de caractres
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void LcdBigString(char *Chaine)
{
 while (*Chaine) LcdBigDigit(*Chaine++ - '0');
}

/*---------------------------------------------------------------
  Nom          :  ClearLineLcd
  Description  :  Effacer une ligne du LCD
  Arguments    :  X ,Y : position initiale (X:0  83 et Y:0  5)
                  LngLine : longueur de la ligne en pixels
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void ClearLineLcd(char X, char Y, char LngLine)
{
 int i;
 LcdGotoXY(X,Y);
 for (i=0; i<LngLine; i++) LcdSend(0,Lcd_Data);
}

/*---------------------------------------------------------------
  Nom          :  ImageLcd
  Description  :  Afficher une image sur le LCD
  Arguments    :  Aucun
  Valeur renvoye : Aucune.
-----------------------------------------------------------------*/
void ImageLcd(void)
{
 int x,y;
 for (y=0; y<Lcd_Y_res/8+1; y++)
  {
   LcdGotoXY(0,y);
   for (x=0; x<Lcd_X_res; x++) LcdSend(Image_Accueil[x+y*Lcd_X_res],Lcd_Data);
  }
 LcdGotoXY(0,0);
}

/*------------------------------------------------------------------------------------
                                        TIMER A
---------------------------------------------------------------------------------------*/

/*--------------------------------------------------
  Nom          :  InitTimerA_FSK
  Description  :  Initialisation du Timer A pour dmodulation FSK :
                    - TAR : continous up, horloge = ACLK, pas d'int. TAIE
                    - CCR0 : Output Compare
                       - interruptions dclenches par EQU0 valides
                       - TACCR0 affect ds le programme d'int pour obtenir une priode
                         de EQU0 gale  Periode_IntTA0
                    - CCR1 : Output Compare
                       - OUT1 : Mode Reset/Set (Set avec EQU0 et Reset avec EQU1)
                       - TACCR1 affect ds le programme d'int pour obtenir une dure
                         de l'impulsion OUT1 gale  Pulse_OUT1
  Arguments    :  aucun
  Valeur renvoye : aucune.
----------------------------------------------------*/
void InitTimerA_FSK(void)
{
 BCSCTL1|=XTS; //LFXT1 : haute frquence (quartz 4MHz)
 TACTL=TASSEL0|MC0;  // Horloge TAR=ACLK=4MHz,
                     // Up mode modulo TACCR0
                     // Pas d'int TAIE
 TACCTL0=CCIS1|CCIE; // Input capture = GND
                     // CAP=0 : compare mode
                     // OUTMODE=0 (non utilis)
                     // CCIE=1 : interruption EQU0 valide
 TACCR0=Periode_IntTA0-1; //Priodes int TA0 = SMCLK/PeriodeIntTA0 = 6800Hz environ
 TACCTL1=CCIS1|OUTMOD1|OUTMOD0; // Input capture = GND
                     // CAP=0 : compare mode
                     // OUTMODE="011" : output Compare, OUT1 : Set/Reset
 TACCR1=Periode_IntTA0-1-Pulse_OUT1;//Pour produire une impulsion sur OUT1
                                    //de dure Pulse_OUT1
// P1SEL|=BIT2; //Test : P1.2/TA1=OUT1 pour controle  l'oscilloscope
// P1DIR|=BIT2;
}

/*--------------------------------------------------
  Nom          :  InitTimerA_Delay
  Description  :  Initialisation du Timer A pour gnrer des dlais :
                    - TAR : continous up, horloge = FDCO/8, int. TAIE inhibe
                    - CCR0, CCR1 et CCR2 : non utiliss
  Arguments    :  aucun
  Valeur renvoye : aucune.
----------------------------------------------------*/
void InitTimerA_Delay(void)
{
 BCSCTL1&=~XTS; //LFXT1 : basse frquence pour rduire la consommation
 TACTL=TASSEL1|ID1|ID0|MC1;  // Horloge TAR=SMCLK/8=FDCO/8,
                             // mode continu, int TAIE inhibe
 TACCTL0=0 ;//Aucune fonction
 TACCTL2=0 ;//Aucune fonction
 TACCTL1=0 ;//Aucune fonction
// TACCTL1=CCIS1|OUTMOD1|OUTMOD0; // Input capture = GND
                     // CAP=0 : compare mode
                     // OUTMODE="011" : output Compare, OUT1 : Set/Reset
// TACCR1=65536/4 ;//Pour produire un signal de rapport cyclique 1/4
// P1SEL|=BIT2; //Test : P1.2/TA1=OUT1 pour controle  l'oscilloscope
// P1DIR|=BIT2;
}

/*------------------------------------------------------------------------------------
                                        CAN 10 bits
---------------------------------------------------------------------------------------*/

/*----------------------------------------------------
  Nom          :  InitCAN10
  Description  :  Initialisation du CAN 10 bits
                    - ADC10CLK = SMCLK
                    - SHI = OUT1 (impulsion produite par le timer A, voie 1)
                    - Mode : conversion dclenche d'une seule voie
                    - Entre analogique = A1 = P2.1
                    - Dure d'chantillonnage max = 4 ADC10CLK
                    - Rfrence : VCC
  Arguments    :  aucun
  Valeur renvoye : aucune.
------------------------------------------------------*/
void InitCAN10(void)
{
 ADC10CTL0=ADC10SR|REFOUT|REF2_5V|ADC10ON;
                                   //SREF="000" : VR+=VCC et VR-=VSS
                                   //ADC10SHT="00" : sample = 4 ADC10CLK
                                   //ADC10SR=1 : Fchantillonnage=6800Hz<50kHz
                                   //REFOUT=1 : Rfrence interne sortie
                                   //REFBURST=0 : VREF+ permanent
                                   //MSC=0 : conversions commandes par SHI
                                   //REF2_5V=1 : rfrence interne = 2,5V
                                   //REFON=0 : rfrence interne OFF (rduire consommation)
                                   //ADC10ON=1 : ADC10 oprationnel
                                   //ADC10IE=0 : pas d'interruption
                                   //ENC=0 : pour affecter les autres bits de config
 ADC10CTL1=INCH0|SHS0|ADC10DF|ADC10SSEL1|ADC10SSEL0|CONSEQ1;
                                   //INCH="0001" : slection voie A1
                                   //SHS="01" : dclenchement avec OUT1 de Timer A
                                   //ADC10DF=1 : conversion en C2
                                   //ISSH=0 : pas d'inversion de OUT1
                                   //ADC10DIV="000" : Clock=SMCLK/1=FDCO/8
                                   //ADC10SEL="11" : horloge = SMCLK
                                   //CONSEQ="10" : conversion continue 1 voie
 ADC10AE=BIT1;   //Validation entre A1=P2.1
 ADC10CTL0|=ENC; //Activation du ADC10
}

/*------------------------------------------------------------------------------------
                                   CAN srie MAX550
---------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------
  Nom          :  CNA_SendCtl
  Description  :  Envoi du 1 octet de la trame CNA MAX550 (control) via le coupleur SPI
  Arguments    :  data -> octet de controle  transmettre
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void CNA_SendCtl(char Ctl)
{
 // Attendre fin de transmission octet data
 while ((U0TCTL & TXEPT)==0) {};
 P3OUT|= CS_CNA; // Sortie "data" en analogique sur point test CNA
 P3OUT&=~CS_CNA; // CS_CNA="0" : dbut du transfert de la trame de 2 octets
 // Transmission srie de l'octet
 U0TXBUF=Ctl;
}

/*---------------------------------------------------------------
  Nom          :  CNA_SendData
  Description  :  Envoi du 2 octet de la trame CNA MAX550 (data) via le coupleur SPI
  Arguments    :  data -> octet data  transmettre
  Valeur renvoye : aucune.
-----------------------------------------------------------------*/
void CNA_SendData(char Data)
{
 // Attendre fin de transmission octet control
 while ((U0TCTL & TXEPT)==0) {};
 // Transmission srie de l'octet
 U0TXBUF=Data;
}

/*------------------------------------------------------------------------------------
                                      UART
---------------------------------------------------------------------------------------*/
/*-------------------------------------------------
  Nom          :  InitUART
  Description  :  Initialisation du coupleur UART :
                   - 8 bits
                   - 1 bit STOP
                   - pas de bit de parit
  Arguments    :  Vitesse de transmission en bauds
  Valeur renvoye : aucune.
---------------------------------------------------*/
// Tableau de configuation pour une horloge ACLK de 32768Hz
const char Tab_BaudRate[4][2]={{0x1B,0x03}, //U0BR0=1Bh, U0MCTL=03h : 1200 bauds
                               {0x0D,0x6B}, //U0BR0=0Dh, U0MCTL=6Bh : 2400 bauds
                               {0x06,0x6F}, //U0BR0=06h, U0MCTL=6Fh : 4800 bauds
                               {0x0D,0x6B}};//U0BR0=03h, U0MCTL=4Ah : 9600 bauds
void InitUART(BaudRate Bauds)
{
  U0CTL=SWRST; //SWRST = "1" pour configurer l'USART en mode UART
  U0CTL=CHAR;  //PE=0 : pas de parit
               //PEV=0
               //SPB=0 : 1 bit Stop
               //CHAR=1 : 8 bits
               //LISTEN=0 : pas de loopback
               //SYNC=0 : mode UART
               //MM=0
  U0TCTL=(U0TCTL & TXEPT)|SSEL0;
               //CKPL=0 : horloge UCLK non inverse
               //SSEL="01" : BRCLK=ACLK
               //URXSE=0 : pas de fonction "Start-edge"
               //TXWAKE=0
  U0RCTL=0;    //Indicateurs FE,PE,OE, BRK et RXERR mis  0
               //URXEIE=0 : pas d'interruption si erreur en rception
               //URXWIE=0 : tous les caractres reus positionnent URXIFG0
               //RXWAKE=0
  U0BR0=Tab_BaudRate[Bauds][0]; //Baud rate
  U0BR1=0;                      //Octet de poids fort tjrs  0
  U0MCTL=Tab_BaudRate[Bauds][1];//Registre de modulation
  ME2|=URXE0;    //Validation rception UART dans TIM_A_Int
  U0CTL&=~SWRST; //SWRST = "0" fin de configuration dans TIM_A_Int
  P3SEL=BIT5;    //P3.5=URXD0
  IFG2=0; //Raz des indicateurs d'interruption,
          //notamment URXIFG0 et UTXIFG0
//  IE2=URXIE0; //Interruptions rception valides
}

/*-------------------------------------------------------------------------
  Nom          :  InitRS232
  Description  :  Initialisation du coupleur UART et des variables associes
  Arguments    :  Vitesse de transmission en bauds
  Valeur renvoye : aucune.
---------------------------------------------------------------------------*/
void InitRS232(BaudRate Bauds)
{
  int i;
  RXWriteIndex = RXReadIndex = RXCharCount = Lng_Message = Count_Bit = 0;
  for (i=0;i<RXBufSize;i++) RXBuffer[i]=0;
  InitUART(Bauds);//Initialisation UART : 8 bits, 1 bit Stop
                  //Interruptions en rception valides
}

/*---------------------------------------------------------------
  Nom         : ReadCharUART
  Description : Lecture du prochain caractre reu sur URXD0
  Arguments   : Aucun
  Retour      : Prochain caractre reu sur URXD s'il existe
                sinon : 0
-----------------------------------------------------------------*/
char ReadCharUART(void)
{
  char c;
  if (RXCharCount) // Un ou plusieurs caractres reus ?
   {
     c = RXBuffer[RXReadIndex++]; // Lecture du prochain caractre
                                  // et incrmentation de l'index lecture
     RXReadIndex &= RXBufSize-1; // Modulo RXBufSize
     RXCharCount--;  // Dcrmentation compteur d'octets
     return(c);
   }
  else return(0); // if there is no new char
}

/*----------------------------------------------------
  Nom          :  Init_Var
  Description  :  Initialisation des variables
  Arguments    :  aucun
  Valeur renvoye : aucune.
------------------------------------------------------*/
void Init_Var(void)
{
 int i;
 for (i=0;i<5;i++) WDF_PARMS[i]=0;
 RXWriteIndex = RXReadIndex = RXCharCount = Lng_Message = Count_Bit = 0;
 for (i=0;i<RXBufSize;i++) RXBuffer[i]=0;
 R6_T=0;
 Compt_Int_6800=0; // Raz du compteur d'interruptions
}

/*------------------------------------------------------------------------------------
                                       Programme principal
--------------------------------------------------------------------------------------*/
void main(void)
{
 WDTCTL=WDTPW+WDTHOLD;       // Arrt du chien de garde
 // Horloge basse
 SetDCOClock(3,1); // FDCO = 190kHz typique
 // Initialisations ports E/S
 P1DIR=TA1; // Port 1
 P2DIR=Lcd_RST_Pin|PWR_ON; // Port 2
 P3DIR=Lcd_CE_Pin|DIN_SPI|CLK_SPI|Lcd_DC_Pin|RXD|CS_CNA|LDAC;//Port 3
 P3OUT=Lcd_CE_Pin|CS_CNA|LDAC;// Controleur LCD et CNA non slectionns
 P2OUT=Lcd_RST_Pin; // Pas de reset (RST = "1") et PWR_ON = "0"
 // Dtection sonnerie
 P2IE |=SONNE; //Validation interruption sur P2.5
 P2IES|=SONNE; //Sur flanc descendant
 // Initialisation USART en mode SPI (pour affichage initial)
 InitSPI_Lcd();//Initialisation du coupleur SPI pour Lcd
 LcdInit();    //Initialisation controleur Lcd
 ImageLcd();   //Ecran d'accueil
 InitTimerA_Delay();// Initialisation TimerA pour raliser monostable (int inhibe)
                    // Horloge TAR = FDCO/8 = 23750Hz typique
 InitCAN10();//Initialisation CAN 10 bits (mais rfrence interne OFF pour consommation)
 IND_R4=0;
 Compt_Sonnerie=0; // Raz du compteur de priodes de sonnerie
 Premier_Numero=0;
 // Validation interruptions ds CPU
 _EINT();
/*--------------------
   Boucle principale
 ---------------------*/
 do
  {
   //P3OUT^=RXD ;// Test : inversion du bit RXD=P3.7
   /* Arrt CPU hors interruptions
      Low power mode 4 (LPM4) si sonnerie non dtecte
      Low power mode 1 (LPM1) entre dtection sonnerie et rception trame CLIP */
   if (IND_R4==0)
   {
    if (TACTL & TAIE) __low_power_mode_1();// CPU et MCLK off si dtection sonnerie
                 else __low_power_mode_4();// Tout  l'arrt
   }
   if (IND_R4 & NEW_MSG)
   {
    // Passer en mode "basse consommation" et affichage du numro appelant
    _DINT();
    SetDCOClock(3,1);  // Horloge basse : FDCO = 190kHz typique
    #define FDCO_low 190000
    P2OUT&=~PWR_ON;    // Arrt alimentation Aop
    ADC10CTL0&=~ENC;   // Pour affecter REFON
    ADC10CTL0&=~REFON; // Arrt rfrence pour rduire la consommation
//    ADC10CTL0|=ENC;    // ADC10 oprationnel
    InitTimerA_Delay();// Initialisation TimerA pour raliser monostable (int inhibe)
                       // Horloge TAR = FDCO/8 = 23750Hz typique
   // Controleur LCD slectionn (CE_LCD = "0")
   P3OUT &= ~Lcd_CE_Pin ;
    if (!(IND_R4 & TIME_OUT))
     {
      if (!Premier_Numero)
       {
         Fill_Lcd(Blanc);  // Effacer l'cran
         LcdString(0,0,"Derniers num.:");
         Pos_Y_Msg=1;     // Position Y initiale des messages
         Premier_Numero=1;
       }
      RXBuffer[Lng_Message-1]=0;
      LcdGotoXY(0,Pos_Y_Msg); LcdChar(' ');// Effacer indicateur
                                           // du dernier numro
      Pos_Y_Msg++; // Position suivante
      if (Pos_Y_Msg > 5) Pos_Y_Msg=1;
      ClearLineLcd(6,Pos_Y_Msg,Lcd_X_res-6-1);//Effacer la ligne
      LcdGotoXY(0,Pos_Y_Msg); LcdChar('#');//Indicateur du dernier numro
      LcdString(6,Pos_Y_Msg,RXBuffer+4);
     }
    IND_R4=0;
    Compt_Sonnerie=0; // Raz du compteur de priodes de sonnerie
    _EINT();
   }
  }
 while (1);
}

/*------------------------------------------------------------------------------------
                                     Programmes d'interruption
--------------------------------------------------------------------------------------*/
/*-----------------------
   Interruption Port P2
 ------------------------
 Active  chaque flanc descendant sur P2.5=SONNE :
   - Prochaine interruption TAI ds 500mS (monostable redclenchable)
     (dure mini phase B de la transmission) */
 #pragma vector=PORT2_VECTOR
 __interrupt void Int_PA2()
 {
  Compt_Sonnerie++;// Incrmentation du compteur de priodes de sonnerie
  TAR=0xFFFF-FDCO_low/16; // Prochaine interruption TAI ds 500mS
  TACTL&=~TAIFG;   // Raz indicateur TAIFG
  TACTL|=TAIE;     // Validation interruption TAI
  P2IFG&=~SONNE;   // Acquitement interruption
  __low_power_mode_off_on_exit();
 }

/*-------------------
   Interruption TAI
 --------------------
 Active 500mS aprs arrt sonnerie puis inhibe
  - Activer la dmodulation FSK par interruptions TimerA au retour d'int. */
#pragma vector=TIMERA1_VECTOR
__interrupt void Int_TAI()
{
 if (Compt_Sonnerie>10) // Nb de priodes de sonnerie suffisants
  {
   // Alimentation Aop
   P2OUT|=PWR_ON;
   // Horloge haute
   SetDCOClock(3,6); // FDCO = 1,9MHz typique
   ADC10CTL0&=~ENC;  //Pour affecter REFON
   ADC10CTL0|=REFON; //Activer la rfrence interne
   ADC10CTL0|=ENC;   //ADC10 oprationnel
   // Controleur LCD non slectionn (CE_LCD = "1")
   P3OUT |= Lcd_CE_Pin ;
   // Initialisation TimerA pour dmodulation FSK
   InitTimerA_FSK(); //Initialisation du Timer A (interruptions 6800Hz)
   // Initialisation USART en mode UART (pour rception asynchrone)
   InitRS232(B1200);//Initialisation UART : 1200 bauds, 8 bits, 1 bit Stop
                    //Interruptions en rception non valides
   // Initialisations variables pour acquisition trame CLIP
   Init_Var();      //Initialisation des variables pour dmod FSK
   TACTL&=~TAIE;    //Inhibition prochaines interruptions
   IND_R4=WAIT_SMMR;//Phase d'attente du signal SMMR
  }
 TACTL&=~TAIFG;     //Raz indicateur TAIFG (acquitement interruption)
 __low_power_mode_off_on_exit();
}
